"""
SAPHostProfile - File ``/usr/sap/hostctrl/exe/host_profile``
============================================================

Shared parser for parsing the ``/usr/sap/hostctrl/exe/host_profile`` file.

"""
from .. import Parser, parser, LegacyItemAccess, get_active_lines, add_filter
from insights.parsers import SkipException
from insights.specs import Specs

filter_list = [
        'SAPSYSTEM', 'DIR_',
]
add_filter(Specs.sap_host_profile, filter_list)


@parser(Specs.sap_host_profile)
class SAPHostProfile(Parser, LegacyItemAccess):
    """
    Class for parsing the `/usr/sap/hostctrl/exe/host_profile` file.

    Typical content of the file is::

        SAPSYSTEMNAME = SAP
        SAPSYSTEM = 99
        service/porttypes = SAPHostControl SAPOscol SAPCCMS
        DIR_LIBRARY = /usr/sap/hostctrl/exe
        DIR_EXECUTABLE = /usr/sap/hostctrl/exe
        DIR_PROFILE = /usr/sap/hostctrl/exe
        DIR_GLOBAL = /usr/sap/hostctrl/exe
        DIR_INSTANCE = /usr/sap/hostctrl/exe
        DIR_HOME = /usr/sap/hostctrl/work

    Examples:
        >>> type(hpf)
        <class 'insights.parsers.sap_host_profile.SAPHostProfile'>
        >>> hpf['SAPSYSTEMNAME']
        'SAP'
        >>> hpf['DIR_HOME']
        '/usr/sap/hostctrl/work'
    """

    def parse_content(self, content):
        self.data = {}
        for line in get_active_lines(content):
            if '=' not in line:
                raise SkipException("Incorrect line: '{0}'".format(line))
            key, val = line.split('=', 1)
            self.data[key.strip()] = val.strip()



from insights.parsers.hponcfg import HponConf
from insights.tests import context_wrap

HPONCFG = """
HP Lights-Out Online Configuration utility
Version 4.3.1 Date 05/02/2014 (c) Hewlett-Packard Company, 2014
Firmware Revision = 1.40 Device type = iLO 4 Driver name = hpilo
Host Information:
                        Server Name: foo.example.com
                        Server Number:
"""


def test_hponcfg():

    conf = HponConf(context_wrap(HPONCFG))

    assert "1.40" == conf.firmware_revision
    assert "iLO 4" == conf.device_type
    assert "hpilo" == conf.driver_name
    assert "foo.example.com" == conf['server_name']
    assert "" == conf['server_number']



"""
IPCS commands
=============

Shared parsers for parsing output of the ``ipcs`` commands.

IpcsM - command ``ipcs -m``
---------------------------

IpcsMP - command ``ipcs -m -p``
-------------------------------

IpcsS - command ``ipcs -s``
---------------------------

IpcsSI - command ``ipcs -s -i {semaphore ID}``
----------------------------------------------

"""
from insights import parser, get_active_lines, CommandParser
from insights.parsers import parse_delimited_table, ParseException, SkipException
from insights.specs import Specs


class IPCS(CommandParser):
    """
    Base class for parsing the output of `ipcs -X` command in which the `X`
    could be `m`, `s` or `q`.

    """

    def parse_content(self, content):
        # heading_ignore is first line we _don't_ want to ignore...
        ids = ['semid', 'shmid', 'msqid']
        table = parse_delimited_table(content, heading_ignore=['key'] + ids)
        if not table:
            raise SkipException('Nothing to parse.')
        id_s = [i for i in table[0] if i in ids]
        if not id_s or len(id_s) != 1:
            raise ParseException('Unexpected heading line.')
        id_ok = id_s[0]
        self.data = {}
        for item in table:
            self.data[item.pop(id_ok)] = item

    def __contains__(self, sid):
        """
        Check if ``sid`` is contained or not
        """
        return sid in self.data

    def __getitem__(self, sid):
        """
        Retrieves an item from the underlying data dictionary.
        """
        return self.data[sid]

    def get(self, sid, default=None):
        """Returns value of key ``item`` in self.data or ``default``
        if key is not present.

        Parameters:
            sid(str): Key to get from ``self.data``.
            default (str): Default value to return if key is not present.

        Returns:
            {dict}: the stored dict item, or the ``default`` if not found.

        """
        return self.data.get(sid, default)


@parser(Specs.ipcs_m)
class IpcsM(IPCS):
    """
    Class for parsing the output of `ipcs -m` command.

    Typical output of the command is::

        ------ Shared Memory Segments --------
        key        shmid      owner      perms      bytes      nattch     status
        0x0052e2c1 0          postgres   600        37879808   26

    Examples:
        >>> '0' in shm
        True
        >>> shm.get('0', {}).get('bytes')
        '37879808'
        >>> '2602' in shm
        False
        >>> shm.get('2602', {}).get('bytes')

    """
    pass


@parser(Specs.ipcs_m_p)
class IpcsMP(IPCS):
    """
    Class for parsing the output of `ipcs -m -p` command.

    Typical output of the command is::

        ------ Shared Memory Creator/Last-op --------
        shmid      owner      cpid       lpid
        0          postgres   1833       14111

    Examples:
        >>> '0' in shmp
        True
        >>> shmp.get('0').get('cpid')
        '1833'
    """
    pass


@parser(Specs.ipcs_s)
class IpcsS(IPCS):
    """
    Class for parsing the output of `ipcs -s` command.

    Typical output of the command is::

        ------ Semaphore Arrays --------
        key        semid      owner      perms      nsems
        0x00000000 557056     apache     600        1
        0x00000000 589825     apache     600        1
        0x00000000 131074     apache     600        1
        0x0052e2c1 163843     postgres   600        17
        0x0052e2c2 196612     postgres   600        17
        0x0052e2c3 229381     postgres   600        17
        0x0052e2c4 262150     postgres   600        17
        0x0052e2c5 294919     postgres   600        17
        0x0052e2c6 327688     postgres   600        17
        0x0052e2c7 360457     postgres   600        17
        0x00000000 622602     apache     600        1
        0x00000000 655371     apache     600        1
        0x00000000 688140     apache     600        1

    Examples:
        >>> '622602' in sem
        True
        >>> '262150' in sem
        True
        >>> sem.get('262150').get('owner')
        'postgres'
    """
    pass


@parser(Specs.ipcs_s_i)
class IpcsSI(CommandParser):
    """
    Class for parsing the output of `ipcs -s -i ##` command. ``##`` will be
    replaced with specific semid

    Typical output of the command is::

        # ipcs -s -i 65536

        Semaphore Array semid=65536
        uid=500  gid=501     cuid=500    cgid=501
        mode=0600, access_perms=0600
        nsems = 8
        otime = Sun May 12 14:44:53 2013
        ctime = Wed May  8 22:12:15 2013
        semnum     value      ncount     zcount     pid
        0          1          0          0          0
        1          1          0          0          0
        0          1          0          0          6151
        3          1          0          0          2265
        4          1          0          0          0
        5          1          0          0          0
        0          0          7          0          6152
        7          1          0          0          4390

    Examples:
        >>> semi.semid
        '65536'
        >>> semi.pid_list
        ['0', '2265', '4390', '6151', '6152']

    """

    def parse_content(self, content):
        # parse the output of `ipcs -s -i ##` command
        pids = set()
        self._semid = None
        for line in get_active_lines(content):
            line = line.strip()
            if line.startswith('Semaphore'):
                self._semid = line.split('=')[-1]
            elif self._semid and line[0].isdigit():
                pids.add(line.split()[-1])
        self._pids = sorted(list(pids))

    @property
    def semid(self):
        """
        Return the semaphore ID.

        Returns:
            str: the semaphore ID.
        """
        return self._semid

    @property
    def pid_list(self):
        """
        Return the ID list of the processes which use this semaphore.

        Returns:
            [list]: the processes' ID list
        """
        return self._pids



from insights.tests import context_wrap
from insights.parsers.greenboot_status import GreenbootStatus


GREEN = """
Boot Status is GREEN - Health Check SUCCESS
"""

RED = """
Mar 04 15:47:12 example greenboot[768]: Script 'check-dns.sh' SUCCESS
Mar 04 15:47:12 example required-services.sh[999]: active
Mar 04 15:47:12 example required-services.sh[999]: active
Mar 04 15:47:12 example required-services.sh[999]: inactive
Mar 04 15:47:10 example NetworkManager[886]: <info>  [1614872830.0295] manager: NetworkManager state is now CONNECTED_GLOBAL
Mar 04 15:47:12 example check-dns.sh[801]: PING 192.168.81.1 (192.168.81.1) 56(84) bytes of data.
Mar 04 15:47:12 example check-dns.sh[801]: 64 bytes from 192.168.81.1: icmp_seq=1 ttl=64 time=0.253 ms
Mar 04 15:47:12 example check-dns.sh[801]: --- 192.168.81.1 ping statistics ---
Mar 04 15:47:12 example check-dns.sh[801]: 1 packets transmitted, 1 received, 0% packet loss, time 0ms
Mar 04 15:47:12 example check-dns.sh[801]: rtt min/avg/max/mdev = 0.253/0.253/0.253/0.000 ms
Mar 04 15:47:12 example greenboot[768]: Script 'check-dns.sh' SUCCESS
Mar 04 15:47:12 example required-services.sh[999]: active
Mar 04 15:47:12 example required-services.sh[999]: active
Mar 04 15:47:12 example required-services.sh[999]: inactive
Mar 04 15:47:12 example greenboot[768]: Script 'required-services.sh' FAILURE (exit code '3')
Mar 04 15:47:12 example systemd[1]: greenboot-healthcheck.service: Main process exited, code=exited, status=3/NOTIMPLEMENTED
Mar 04 15:47:12 example systemd[1]: greenboot-healthcheck.service: Failed with result 'exit-code'.
Mar 04 15:47:12 example systemd[1]: Failed to start greenboot Health Checks Runner.
Mar 04 15:47:12 example systemd[1]: Dependency failed for Boot Completion Check.
Mar 04 15:47:12 example systemd[1]: Dependency failed for Mark boot as successful in grubenv.
Mar 04 15:47:12 example systemd[1]: Dependency failed for Multi-User System.
Mar 04 15:47:12 example systemd[1]: multi-user.target: Job multi-user.target/start failed with result 'dependency'.
Mar 04 15:47:12 example systemd[1]: greenboot-grub2-set-success.service: Job greenboot-grub2-set-success.service/start failed with result 'dependency'.
Mar 04 15:47:12 example systemd[1]: Dependency failed for greenboot Success Scripts Runner.
Mar 04 15:47:12 example systemd[1]: greenboot-task-runner.service: Job greenboot-task-runner.service/start failed with result 'dependency'.
Mar 04 15:47:12 example systemd[1]: boot-complete.target: Job boot-complete.target/start failed with result 'dependency'.
Mar 04 15:47:12 example systemd[1]: greenboot-healthcheck.service: Triggering OnFailure= dependencies.
Mar 04 15:47:12 example systemd[1]: Starting greenboot Failure Scripts Runner...
Mar 04 15:47:12 example systemd[1]: Starting Update UTMP about System Runlevel Changes...
Mar 04 15:47:12 example greenboot[1004]: Boot Status is RED - Health Check FAILURE!
Mar 04 15:47:12 example greenboot[1004]: Running Red Scripts...
Mar 04 15:47:12 example systemd[1]: Started greenboot Failure Scripts Runner.
Mar 04 15:47:12 example systemd[1]: Starting Reboot on red boot status...
Mar 04 15:47:12 example systemd[1]: Starting greenboot MotD Generator...
Mar 04 15:47:12 example systemd[1]: Reached target Generic red boot target.
Mar 04 15:47:12 example redboot-auto-reboot[1009]: SYSTEM is UNHEALTHY, but boot_counter is unset in grubenv. Manual intervention necessary.
Mar 04 15:47:12 example systemd[1]: systemd-update-utmp-runlevel.service: Succeeded.
Mar 04 15:47:12 example systemd[1]: Started Update UTMP about System Runlevel Changes.
Mar 04 15:47:12 example systemd[1]: redboot-auto-reboot.service: Main process exited, code=exited, status=1/FAILURE
Mar 04 15:47:12 example systemd[1]: redboot-auto-reboot.service: Failed with result 'exit-code'.
Mar 04 15:47:12 example systemd[1]: Failed to start Reboot on red boot status.
Mar 04 15:47:12 example greenboot-status[1010]: Script 'required-services.sh' FAILURE (exit code '3')
Mar 04 15:47:12 example greenboot-status[1010]: Boot Status is RED - Health Check FAILURE!
Mar 04 15:47:12 example greenboot-status[1010]: SYSTEM is UNHEALTHY, but boot_counter is unset in grubenv. Manual intervention necessary.
Mar 04 15:47:12 example systemd[1]: Started greenboot MotD Generator.
"""

FALLBACK = """
Feb 22 22:50:26 example systemd[1]: Starting greenboot MotD Generator...
Feb 22 22:50:26 example greenboot-status[905]: Boot Status is GREEN - Health Check SUCCESS
Feb 22 22:50:26 example greenboot-status[905]: FALLBACK BOOT DETECTED! Default rpm-ostree deployment has been rolled back.
Feb 22 22:50:26 example systemd[1]: Started greenboot MotD Generator.
"""


def test_greenboot_status_green():
    green = context_wrap(GREEN)
    p = GreenbootStatus(green)
    assert p.green
    assert not p.red


def test_greenboot_status_red():
    red = context_wrap(RED)
    p = GreenbootStatus(red)
    assert p.red
    assert not p.green


def test_greenboot_status_fallback():
    fb = context_wrap(FALLBACK)
    p = GreenbootStatus(fb)
    assert p.green
    assert p.fallback



"""
GrubConf - The valid GRUB configuration
=======================================
Combiner for Red Hat Grub v1 Grub v2, and BLS information.

This combiner uses the parsers:
:class:`insights.parsers.grub_conf.Grub1Config`,
:class:`insights.parsers.grub_conf.Grub1EFIConfig`,
:class:`insights.parsers.grub_conf.Grub2Config`,
:class:`insights.parsers.grub_conf.Grub2EFIConfig`, and
:class:`insights.parsers.grub_conf.BootLoaderEntries`.
:class:`insights.parsers.grub_env.GrubEnv`.

It determines which parser was used by checking one of the following
parsers/combiners:
:class:`insights.parsers.installed_rpms.InstalledRpms`,
:class:`insights.parsers.cmdline.CmdLine`,
:class:`insights.parsers.ls_sys_firmware.LsSysFirmware`, and
:class:`insights.combiners.redhat_release.RedHatRelease`.
"""
import re

from insights import SkipComponent
from insights.combiners.redhat_release import RedHatRelease
from insights.core.plugins import combiner
from insights.parsers.cmdline import CmdLine
from insights.parsers.grub_conf import (get_kernel_initrds, BootEntry, Grub1Config, Grub1EFIConfig, Grub2Config,
                                        Grub2EFIConfig, BootLoaderEntries as BLE)
from insights.parsers.grubenv import GrubEnv
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.ls_sys_firmware import LsSysFirmware


@combiner(BLE, optional=[GrubEnv, LsSysFirmware])
class BootLoaderEntries(object):
    """
    Combine all :class:`insights.parsers.grub_conf.BootLoaderEntries`
    parsers into one Combiner

    Attributes:
        version (int): The version of the GRUB configuration, 1 or 2
        is_efi (bool): If the host is boot with EFI
        entries (list): List of all boot entries in GRUB configuration.
        boot_entries (list): List of all boot entries which only contains the
            name and cmdline.
        kernel_initrds (dict): Dict of the `kernel` and `initrd` files
            referenced in GRUB configuration files
        is_kdump_iommu_enabled (bool): If any kernel entry contains "intel_iommu=on"

    Raises:
        SkipComponent: when no any BootLoaderEntries Parsers.
    """
    def __init__(self, grub_bles, grubenv, sys_firmware):
        self.version = self._version = 2
        self.is_efi = self._efi = '/sys/firmware/efi' in sys_firmware if sys_firmware else False
        self.entries = []
        self.boot_entries = []
        self.is_kdump_iommu_enabled = False

        for ble in grub_bles:
            # Make a copy of the ble entry, so that no write
            # backs occur below when expanding variables.
            self.entries.append(ble.entry.copy())
            self.boot_entries.append(BootEntry({'name': ble.title, 'cmdline': ble.cmdline,
                                                'version': ble.entry.get('version')}))
            self.is_kdump_iommu_enabled = self.is_kdump_iommu_enabled or ble.is_kdump_iommu_enabled

        # If grub_bles and grubenv expand the $kernelopts,
        # $tuned_params, and $tuned_initrd variables.
        if grub_bles and grubenv:
            for entry in self.entries:
                entry_options = entry.get('options', "")
                if "$kernelopts" in entry_options or "$tuned_params" in entry_options:
                    entry['options'] = re.sub("\\$kernelopts", grubenv.kernelopts,
                                              entry['options']).strip()
                    entry['options'] = re.sub("\\$tuned_params", grubenv.tuned_params,
                                              entry['options']).strip()

                if "$tuned_initrd" in entry.get('initrd', "") and grubenv.get('tuned_initrd'):
                    entry['initrd'] = re.sub("\\$tuned_initrd", grubenv.get('tuned_initrd', ""),
                                             entry['initrd']).strip()

            for entry in self.boot_entries:
                entry_options = entry.get('cmdline', "")
                if "$kernelopts" in entry_options or "$tuned_params" in entry_options:
                    entry['cmdline'] = re.sub("\\$kernelopts", grubenv.kernelopts, entry['cmdline']).strip()
                    entry['cmdline'] = re.sub("\\$tuned_params", grubenv.tuned_params, entry['cmdline']).strip()

        self.kernel_initrds = get_kernel_initrds(self.entries)

        if not self.entries:
            raise SkipComponent()


@combiner([Grub1Config, Grub2Config, Grub1EFIConfig, Grub2EFIConfig, BootLoaderEntries],
          optional=[InstalledRpms, CmdLine, LsSysFirmware, RedHatRelease])
class GrubConf(object):
    """
    Process Grub configuration v1, v2, and BLS based on which type is passed in.

    Attributes:
        version (int): returns 1 or 2, version of the GRUB configuration
        is_efi (bool): returns True if the host is boot with EFI
        kernel_initrds (dict): returns a dict of the `kernel` and `initrd`
            files referenced in GRUB configuration files
        is_kdump_iommu_enabled (bool): returns True if any kernel entry
            contains "intel_iommu=on"

    Raises:
        Exception: when cannot find any valid grub configuration.

    Examples:
        >>> type(grub_conf)
        <class 'insights.combiners.grub_conf.GrubConf'>
        >>> grub_conf.version
        2
        >>> grub_conf.is_efi
        False
        >>> grub_conf.kernel_initrds
        {'grub_initrds': ['/initramfs-3.10.0-327.36.3.el7.x86_64.img'],
         'grub_kernels': ['/vmlinuz-3.10.0-327.36.3.el7.x86_64']}
        >>> grub_conf.is_kdump_iommu_enabled
        False
        >>> grub_conf.get_grub_cmdlines('')
        []
    """
    def __init__(self, grub1, grub2, grub1_efi, grub2_efi, grub_bles,
                 rpms, cmdline, sys_firmware, rh_rel):
        self.version = self.is_kdump_iommu_enabled = None
        self.grub = self.kernel_initrds = None
        self.is_efi = '/sys/firmware/efi' in sys_firmware if sys_firmware else False
        _grubs = list(filter(None, [grub1, grub2, grub1_efi, grub2_efi, grub_bles]))

        if len(_grubs) == 1:
            self.grub = _grubs[0]
            self.is_efi = self.is_efi if sys_firmware else self.grub._efi
        else:
            _grub1, _grub2 = (grub1_efi, grub2_efi) if self.is_efi else (grub1, grub2)
            if rh_rel and rh_rel.rhel8:
                self.grub = grub_bles
            # Check grub version via installed-rpms
            else:
                if rpms:
                    # grub1
                    if 'grub2' not in rpms and 'grub' in rpms and _grub1 is not None:
                        self.grub = _grub1
                    # grub2
                    if 'grub' not in rpms and 'grub2' in rpms and _grub2 is not None:
                        self.grub = _grub2
                # Check grub version via the booted CmdLine
                if self.grub is None and cmdline:
                    # grub1
                    if "BOOT_IMAGE" not in cmdline or 'rd_LVM_LV' in cmdline:
                        self.grub = _grub1
                    # grub2
                    if "BOOT_IMAGE" in cmdline or 'rd.lvm.lv' in cmdline:
                        self.grub = _grub2

        if self.grub:
            self.version = self.grub._version
            self.is_kdump_iommu_enabled = self.grub.is_kdump_iommu_enabled
            self.kernel_initrds = self.grub.kernel_initrds or {}
        else:
            raise Exception('No valid grub configuration is found.')

    def get_grub_cmdlines(self, search_text=None):
        """
        Get the boot entries in which `cmdline` contains the `search_text`,
        return all the boot entries by default.

        Arguments:
            search_text(str): keyword to find in the `cmdline`, being set to
                None by default.

        Returns:
            A list of :class:`insights.parsers.grub_conf.BootEntry` objects fo
            each boot entry in which the `cmdline` contains the `search_text`.
            When `search_text` is None, returns the objects of all of the boot entries.
        """
        if search_text is None:
            return self.grub.boot_entries
        elif search_text:
            return [e for e in self.grub.boot_entries if search_text in e.get('cmdline', '')]
        return []



import doctest

from insights.parsers import ls_var_run
from insights.parsers.ls_var_run import LsVarRun
from insights.tests import context_wrap

LS_VAR_RUN = """
total 20
drwx--x---.  2   0 984   40 May 15 09:29 openvpn
drwxr-xr-x.  2   0   0   40 May 15 09:30 plymouth
drwxr-xr-x.  2   0   0   40 May 15 09:29 ppp
drwxr-xr-x.  2  75  75   40 May 15 09:29 radvd
-rw-r--r--.  1   0   0    5 May 15 09:30 rhnsd.pid
drwxr-xr-x.  2   0   0   60 May 30 09:31 rhsm
drwx------.  2  32  32   40 May 15 09:29 rpcbind
-r--r--r--.  1   0   0    0 May 17 16:26 rpcbind.lock
"""


def test_ls_var_run():
    ls_var_run = LsVarRun(context_wrap(LS_VAR_RUN, path='insights_commands/ls_-lnL_.var.run'))
    assert ls_var_run.dirs_of('/var/run') == ['openvpn', 'plymouth', 'ppp', 'radvd', 'rhsm', 'rpcbind']
    foreman = ls_var_run.dir_entry('/var/run', 'openvpn')
    assert foreman is not None
    assert foreman == {
        'group': '984',
        'name': 'openvpn',
        'links': 2,
        'perms': 'rwx--x---.',
        'raw_entry': 'drwx--x---.  2   0 984   40 May 15 09:29 openvpn',
        'owner': '0',
        'date': 'May 15 09:29',
        'type': 'd',
        'size': 40,
        'dir': '/var/run'}


def test_ls_var_run_doc_examples():
    env = {
        'LsVarRun': LsVarRun,
        'ls_var_run': LsVarRun(context_wrap(LS_VAR_RUN, path='insights_commands/ls_-lnL_.var.run')),
    }
    failed, total = doctest.testmod(ls_var_run, globs=env)
    assert failed == 0



"""
RsyslogConf - file ``/etc/rsyslog.conf``
========================================

The rsyslog configuration files can include statements with two different
line based formats along with snippets of 'RainerScript' that can span
multiple lines.

See http://www.rsyslog.com/doc/master/configuration/basic_structure.html#statement-types

Due to high parsing complexity, this parser presents a simple line-based
view of the file that meets the needs of the current rules.
"""
from .. import Parser, parser, get_active_lines
from insights.specs import Specs
from insights.core.filters import add_filter


add_filter(Specs.rsyslog_conf, ["{", "}", "(", ")"])


@parser(Specs.rsyslog_conf)
class RsyslogConf(Parser, list):
    """
    Parses `/etc/rsyslog.conf` content.

    Skips lines that begin with hash ("#") or are only whitespace.

    Attributes:
        data (list): List of lines in the file that don't start
            with '#' and aren't whitespace.

    Example:
        >>> type(rsysconf)
        <class 'insights.parsers.rsyslog_conf.RsyslogConf'>
        >>> len(rsysconf)
        13
        >>> rsysconf[2]
        'authpriv.*                                              /var/log/secure'
    """
    def __init__(self, *args, **kwargs):
        super(RsyslogConf, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        data = []
        brace_flag = False
        parenthesis_flag = False
        parenthesis_string = ""
        brace_string = ""

        for line in get_active_lines(content):
            l_strip = line.strip()
            # Combine multi lines in brace into one line
            if brace_flag:
                brace_string = brace_string + " " + l_strip
                if "}" in l_strip:
                    data.append(brace_string)
                    brace_string = ""
                    brace_flag = False
                continue
            else:
                if "{" in l_strip:
                    if "}" in l_strip:
                        data.append(l_strip)
                    else:
                        brace_flag = True
                        brace_string = l_strip
                    continue
                # Combine multi lines in parenthesis and not in brace into one line
                if parenthesis_flag:
                    parenthesis_string = parenthesis_string + " " + l_strip
                    if ")" in l_strip:
                        data.append(parenthesis_string)
                        parenthesis_string = ""
                        parenthesis_flag = False
                    continue
                else:
                    if "(" in l_strip:
                        if ")" in l_strip:
                            data.append(l_strip)
                        else:
                            parenthesis_flag = True
                            parenthesis_string = l_strip
                        continue
                    else:
                        data.append(l_strip)
        self.extend(data)



"""
Secure -  file ``/var/log/secure``
==================================
"""
from .. import Syslog, parser
from insights.specs import Specs


@parser(Specs.secure)
class Secure(Syslog):
    """Class for parsing the ``/var/log/secure`` file.

    Sample log text::

        Aug 24 09:31:39 localhost polkitd[822]: Finished loading, compiling and executing 6 rules
        Aug 24 09:31:39 localhost polkitd[822]: Acquired the name org.freedesktop.PolicyKit1 on the system bus
        Aug 25 13:52:54 localhost sshd[23085]: pam_unix(sshd:session): session opened for user zjj by (uid=0)
        Aug 25 13:52:54 localhost sshd[23085]: error: openpty: No such file or directory

    .. note::
        Please refer to its super-class :class:`insights.core.Syslog`

    .. note::
        Because timestamps in the secure log by default have no year,
        the year of the logs will be inferred from the year in your
        timestamp. This will also work around December/January crossovers.

    Examples:
        >>> secure = shared[Secure]
        >>> secure.get('session opened')
        [{'timestamp':'Aug 25 13:52:54',
          'hostname':'localhost',
          'procname': 'sshd[23085]',
          'message': 'pam_unix(sshd:session): session opened for user zjj by (uid=0)',
          'raw_message': 'Aug 25 13:52:54 localhost sshd[23085]: pam_unix(sshd:session): session opened for user zjj by (uid=0)'
         }]
        >>> len(list(secure.get_after(datetime(2017, 8, 25, 0, 0, 0))))
        2
    """
    time_format = '%b %d %H:%M:%S'



"""
XinetdConf - files ``/etc/xinetd.conf`` and in ``/etc/xinetd.d/``
=================================================================

This module provides parsing for the ``/etc/xinetd.conf`` and
``/etc/xinetd.d/*`` files.

Sample input data of file ``/etc/xinetd.conf`` looks like::

    defaults
    {
            enabled                 =
            no_access               = 0.0.0.0/0
            instances               = 60
            per_source              = 128
            log_type                = SYSLOG authpriv
            log_on_success	    = HOST PID DURATION EXIT
            log_on_failure	    = HOST
            cps                     = 25 30
            max_load                = 2
    }

    includedir /etc/xinetd.d

Sample input data of file ``/etc/xinetd.d/tftp`` looks like::

    service tftp
    {
            socket_type             = dgram
            protocol                = udp
            wait                    = yes
            user                    = root
            server                  = /usr/sbin/in.tftpd
            server_args             = -s /var/lib/tftpboot
            disable                 = yes
            per_source              = 11
            cps                     = 100 2
            flags                   = IPv4
    }

Examples:

    >>> xinetd_conf = shared[XinetdConf].data
    >>> assert xinetd_conf.get('is_valid') == True
    >>> assert xinetd_conf.get('is_includedir') == True
    >>> xinetd_conf.get('is_includedir')
    '/etc/xinetd.d'
    >>> 'defaults' in xinetd_conf
    True
    >>> xinetd_conf.get('defaults')
    {   'enabled': '',
        'v6only': 'no',
        'log_on_failure': 'HOST',
        'umask': '002',
        'log_on_success': 'PID HOST DURATION EXIT',
        'instances': '50',
        'per_source': '10',
        'groups': 'yes',
        'cps': '50 10',
        'log_type': 'SYSLOG daemon info'
    }
    >>> 'tftp' in xinetd_conf
    True
    >>> xinetd_conf.get('tftp')
    {	'protocol': 'udp',
        'socket_type': 'dgram',
        'server': '/usr/sbin/in.tftpd',
        'server_args': '-s /var/lib/tftpboot',
        'disable': 'yes',
        'flags': 'IPv4',
        'user': 'root',
        'per_source': '11',
        'cps': '100 2',
        'wait': 'yes'
    }
    >>> 'abc' in xinetd_conf
    False
    >>> xinetd_conf.get('abc')
    >>>
    >>>
    >>> XINETD_CONF_BAD = '''
    ... defaults {
    ...         umask           = 002
    ... }
    ...
    ... includedir /etc/xinetd.d
    ... '''
    >>> xinetd_conf = shared[XinetdConf].data
    >>> assert xinetd_conf.get('is_valid') == False
    >>> 'defaults' in xinetd_conf
    False

"""

from .. import LegacyItemAccess, Parser, parser, get_active_lines
from insights.specs import Specs


@parser(Specs.xinetd_conf)
class XinetdConf(LegacyItemAccess, Parser):
    """Parse contents of file ``/etc/xinetd.conf`` and ``/etc/xinetd.d/*``."""
    def parse_content(self, content):
        self.data = {}
        self.is_valid = True
        section = None
        for line in get_active_lines(content):

            if "{" == line:
                section_con = {}

            elif "}" == line:
                self.data[section] = section_con
                section = None

            elif "includedir" in line:
                self.data["includedir"] = line.split(None, 1)[-1]

            elif not section and "defaults" == line:
                section = line
            elif not section and "service" in line:
                section = line.split(None, 1)[-1]

            elif "=" in line:
                key, _, value = line.partition("=")
                section_con[key.strip()] = value.strip()

            else:
                self.is_valid = False
                break
        self.is_includedir = '/etc/xinetd.d' == self.data.get('includedir')



"""
Module handling HTTP Requests and Connection Diagnostics
"""
from __future__ import print_function
from __future__ import absolute_import
import requests
import os
import six
import json
import logging
import platform
import xml.etree.ElementTree as ET
import warnings
import errno
# import io
from tempfile import TemporaryFile
# from datetime import datetime, timedelta
try:
    # python 2
    from urlparse import urlparse
    from urllib import quote
except ImportError:
    # python 3
    from urllib.parse import urlparse
    from urllib.parse import quote
from .utilities import (determine_hostname,
                        generate_machine_id,
                        write_unregistered_file,
                        write_registered_file,
                        os_release_info)
from .cert_auth import rhsmCertificate
from .constants import InsightsConstants as constants
from .url_cache import URLCache
from insights import package_info
from insights.util.canonical_facts import get_canonical_facts

warnings.simplefilter('ignore')
APP_NAME = constants.app_name
NETWORK = constants.custom_network_log_level
logger = logging.getLogger(__name__)

"""
urllib3's logging is chatty
"""
URLLIB3_LOGGER = logging.getLogger('urllib3.connectionpool')
URLLIB3_LOGGER.setLevel(logging.WARNING)
URLLIB3_LOGGER = logging.getLogger('requests.packages.urllib3.connectionpool')
URLLIB3_LOGGER.setLevel(logging.WARNING)

REQUEST_FAILED_EXCEPTIONS = (requests.ConnectionError, requests.Timeout)

# TODO: Document this, or turn it into a real option
if os.environ.get('INSIGHTS_DEBUG_HTTP'):
    import httplib
    httplib.HTTPConnection.debuglevel = 1
    logging.basicConfig()
    logging.getLogger().setLevel(logging.DEBUG)
    requests_log = logging.getLogger("urllib3")
    requests_log.setLevel(logging.DEBUG)
    requests_log.propagate = True


def _host_not_found():
    raise Exception("Error: failed to find host with matching machine-id. Run insights-client --status to check registration status")


def _api_request_failed(exception, message='The Insights API could not be reached.'):
    logger.error(exception)
    if message:
        logger.error(message)


class InsightsConnection(object):

    """
    Helper class to manage details about the connection
    """

    def __init__(self, config):
        self.config = config
        self.username = self.config.username
        self.password = self.config.password

        # workaround while we support both legacy and plat APIs
        self.cert_verify = self.config.cert_verify
        if self.cert_verify is None:
            # if self.config.legacy_upload:
            self.cert_verify = os.path.join(
                constants.default_conf_dir,
                'cert-api.access.redhat.com.pem')
            # else:
            # self.cert_verify = True
        else:
            if isinstance(self.cert_verify, six.string_types):
                if self.cert_verify.lower() == 'false':
                    self.cert_verify = False
                elif self.cert_verify.lower() == 'true':
                    self.cert_verify = True

        protocol = "https://"

        self.auto_config = self.config.auto_config

        # workaround while we support both legacy and plat APIs
        # hack to "guess" the correct base URL if autoconfig off +
        #   no base_url in config
        if self.config.base_url is None:
            if self.config.legacy_upload:
                self.base_url = protocol + constants.legacy_base_url
            else:
                self.base_url = protocol + constants.base_url
        else:
            self.base_url = protocol + self.config.base_url
        # end hack. in the future, make cloud.redhat.com the default

        self.upload_url = self.config.upload_url
        if self.upload_url is None:
            if self.config.legacy_upload:
                self.upload_url = self.base_url + "/uploads"
            else:
                self.upload_url = self.base_url + '/ingress/v1/upload'

        self.api_url = self.base_url
        self.branch_info_url = self.config.branch_info_url
        if self.branch_info_url is None:
            # workaround for a workaround for a workaround
            base_url_base = self.base_url.split('/platform')[0]
            self.branch_info_url = base_url_base + '/v1/branch_info'
        self.inventory_url = self.api_url + "/inventory/v1"

        self.authmethod = self.config.authmethod
        self.systemid = self.config.systemid or None
        self.get_proxies()
        self.session = self._init_session()

    def _init_session(self):
        """
        Set up the session, auth is handled here
        """
        session = requests.Session()
        session.headers = {'User-Agent': self.user_agent,
                           'Accept': 'application/json'}
        if self.systemid is not None:
            session.headers.update({'systemid': self.systemid})
        if self.authmethod == "BASIC":
            session.auth = (self.username, self.password)
        elif self.authmethod == "CERT":
            cert = rhsmCertificate.certpath()
            key = rhsmCertificate.keypath()
            if rhsmCertificate.exists():
                session.cert = (cert, key)
            else:
                logger.error('ERROR: Certificates not found.')
        session.verify = self.cert_verify
        session.proxies = self.proxies
        session.trust_env = False
        if self.proxy_auth:
            # HACKY
            try:
                # Need to make a request that will fail to get proxies set up
                logger.log(NETWORK, "GET %s", self.base_url)
                session.request(
                    "GET", self.base_url, timeout=self.config.http_timeout)
            except requests.ConnectionError:
                pass
            # Major hack, requests/urllib3 does not make access to
            # proxy_headers easy
            proxy_mgr = session.adapters['https://'].proxy_manager[self.proxies['https']]
            auth_map = {'Proxy-Authorization': self.proxy_auth}
            proxy_mgr.proxy_headers = auth_map
            proxy_mgr.connection_pool_kw['_proxy_headers'] = auth_map
            conns = proxy_mgr.pools._container
            for conn in conns:
                connection = conns[conn]
                connection.proxy_headers = auth_map
        return session

    def _http_request(self, url, method, log_response_text=True, **kwargs):
        '''
        Perform an HTTP request, net logging, and error handling
        Parameters
            url     - URL to perform the request against
            method  - HTTP method, used for logging
            kwargs  - Rest of the args to pass to the request function
        Returns
            HTTP response object
        '''
        logger.log(NETWORK, "%s %s", method, url)
        res = self.session.request(url=url, method=method, timeout=self.config.http_timeout, **kwargs)
        logger.log(NETWORK, "HTTP Status: %d %s", res.status_code, res.reason)
        if log_response_text or res.status_code != 200:
            logger.log(NETWORK, "HTTP Response Text: %s", res.text)
        return res

    def get(self, url, **kwargs):
        return self._http_request(url, 'GET', **kwargs)

    def post(self, url, **kwargs):
        return self._http_request(url, 'POST', **kwargs)

    def put(self, url, **kwargs):
        return self._http_request(url, 'PUT', **kwargs)

    def patch(self, url, **kwargs):
        return self._http_request(url, 'PATCH', **kwargs)

    def delete(self, url, **kwargs):
        return self._http_request(url, 'DELETE', **kwargs)

    @property
    def user_agent(self):
        """
        Generates and returns a string suitable for use as a request user-agent
        """
        import pkg_resources
        core_version = "insights-core"
        pkg = pkg_resources.working_set.find(pkg_resources.Requirement.parse(core_version))
        if pkg is not None:
            core_version = "%s %s" % (pkg.project_name, pkg.version)
        else:
            core_version = "Core %s" % package_info["VERSION"]

        try:
            from insights_client import constants as insights_client_constants
            client_version = "insights-client/{0}".format(insights_client_constants.InsightsConstants.version)
        except ImportError:
            client_version = "insights-client"

        if os.path.isfile(constants.ppidfile):
            with open(constants.ppidfile, 'r') as f:
                parent_process = f.read()
        else:
            parent_process = "unknown"

        requests_version = None
        pkg = pkg_resources.working_set.find(pkg_resources.Requirement.parse("requests"))
        if pkg is not None:
            requests_version = "%s %s" % (pkg.project_name, pkg.version)

        python_version = "%s %s" % (platform.python_implementation(), platform.python_version())

        os_family, os_release = os_release_info()
        kernel_version = "%s %s" % (platform.system(), platform.release())

        ua = "{client_version} ({core_version}; {requests_version}) {os_family} {os_release} ({python_version}; {kernel_version}); {parent_process}".format(
            client_version=client_version,
            core_version=core_version,
            parent_process=parent_process,
            python_version=python_version,
            os_family=os_family,
            os_release=os_release,
            kernel_version=kernel_version,
            requests_version=requests_version,
        )

        return ua

    def get_proxies(self):
        """
        Determine proxy configuration
        """
        # Get proxy from ENV or Config
        proxies = None
        proxy_auth = None
        no_proxy = os.environ.get('NO_PROXY')
        logger.debug("NO PROXY: %s", no_proxy)

        # CONF PROXY TAKES PRECEDENCE OVER ENV PROXY
        conf_proxy = self.config.proxy
        if ((conf_proxy is not None and
             conf_proxy.lower() != 'None'.lower() and
             conf_proxy != "")):
            if '@' in conf_proxy:
                scheme = conf_proxy.split(':')[0] + '://'
                logger.debug("Proxy Scheme: %s", scheme)
                location = conf_proxy.split('@')[1]
                logger.debug("Proxy Location: %s", location)
                username = conf_proxy.split(
                    '@')[0].split(':')[1].replace('/', '')
                logger.debug("Proxy User: %s", username)
                password = conf_proxy.split('@')[0].split(':')[2]
                proxy_auth = requests.auth._basic_auth_str(username, password)
                conf_proxy = scheme + location
            logger.debug("CONF Proxy: %s", conf_proxy)
            proxies = {"https": conf_proxy}

        # HANDLE NO PROXY CONF PROXY EXCEPTION VERBIAGE
        if no_proxy and conf_proxy:
            logger.debug("You have environment variable NO_PROXY set "
                         "as well as 'proxy' set in your configuration file. "
                         "NO_PROXY environment variable will be ignored.")

        # IF NO CONF PROXY, GET ENV PROXY AND NO PROXY
        if proxies is None:
            env_proxy = os.environ.get('HTTPS_PROXY')
            if env_proxy:
                if '@' in env_proxy:
                    scheme = env_proxy.split(':')[0] + '://'
                    logger.debug("Proxy Scheme: %s", scheme)
                    location = env_proxy.split('@')[1]
                    logger.debug("Proxy Location: %s", location)
                    username = env_proxy.split('@')[0].split(':')[1].replace('/', '')
                    logger.debug("Proxy User: %s", username)
                    password = env_proxy.split('@')[0].split(':')[2]
                    proxy_auth = requests.auth._basic_auth_str(username, password)
                    env_proxy = scheme + location
                logger.debug("ENV Proxy: %s", env_proxy)
                proxies = {"https": env_proxy}
            if no_proxy:
                insights_service_host = urlparse(self.base_url).hostname
                logger.debug('Found NO_PROXY set. Checking NO_PROXY %s against base URL %s.', no_proxy, insights_service_host)
                for no_proxy_host in no_proxy.split(','):
                    logger.debug('Checking %s against %s', no_proxy_host, insights_service_host)
                    if no_proxy_host == '*':
                        proxies = None
                        proxy_auth = None
                        logger.debug('Found NO_PROXY asterisk(*) wildcard, disabling all proxies.')
                        break
                    elif no_proxy_host.startswith('.') or no_proxy_host.startswith('*'):
                        if insights_service_host.endswith(no_proxy_host.replace('*', '')):
                            proxies = None
                            proxy_auth = None
                            logger.debug('Found NO_PROXY range %s matching %s', no_proxy_host, insights_service_host)
                            break
                    elif no_proxy_host == insights_service_host:
                        proxies = None
                        proxy_auth = None
                        logger.debug('Found NO_PROXY %s exactly matching %s', no_proxy_host, insights_service_host)
                        break

        self.proxies = proxies
        self.proxy_auth = proxy_auth

    def _legacy_test_urls(self, url, method):
        """
        Actually test the url
        """
        # tell the api we're just testing the URL
        test_flag = {'test': 'test'}
        url = urlparse(url)
        test_url = url.scheme + "://" + url.netloc
        last_ex = None
        paths = (url.path + '/', '', '/r', '/r/insights')
        for ext in paths:
            try:
                logger.log(NETWORK, "Testing: %s", test_url + ext)
                if method == "POST":
                    test_req = self.post(test_url + ext, data=test_flag)
                elif method == "GET":
                    test_req = self.get(test_url + ext)
                # Strata returns 405 on a GET sometimes, this isn't a big deal
                if test_req.status_code in (200, 201):
                    logger.info(
                        "Successfully connected to: %s", test_url + ext)
                    return True
                else:
                    logger.info("Connection failed")
                    return False
            except requests.ConnectionError as exc:
                last_ex = exc
                logger.error(
                    "Could not successfully connect to: %s", test_url + ext)
                print(exc)
        if last_ex:
            raise last_ex

    def _test_urls(self, url, method):
        '''
        Test a URL
        '''
        if self.config.legacy_upload:
            return self._legacy_test_urls(url, method)
        try:
            logger.log(NETWORK, 'Testing %s', url)
            if method == 'POST':
                test_tar = TemporaryFile(mode='rb', suffix='.tar.gz')
                test_files = {
                    'file': ('test.tar.gz', test_tar, 'application/vnd.redhat.advisor.collection+tgz'),
                    'metadata': '{\"test\": \"test\"}'
                }
                test_req = self.post(url, files=test_files)
            elif method == "GET":
                test_req = self.get(url)
            if test_req.status_code in (200, 201, 202):
                logger.info(
                    "Successfully connected to: %s", url)
                return True
            else:
                logger.info("Connection failed")
                return False
        except requests.ConnectionError as exc:
            last_ex = exc
            logger.error(
                "Could not successfully connect to: %s", url)
            print(exc)
        if last_ex:
            raise last_ex

    def test_connection(self, rc=0):
        """
        Test connection to Red Hat
        """
        logger.debug("Proxy config: %s", self.proxies)
        try:
            logger.info("=== Begin Upload URL Connection Test ===")
            upload_success = self._test_urls(self.upload_url, "POST")
            logger.info("=== End Upload URL Connection Test: %s ===\n",
                        "SUCCESS" if upload_success else "FAILURE")
            logger.info("=== Begin API URL Connection Test ===")
            if self.config.legacy_upload:
                api_success = self._test_urls(self.base_url, "GET")
            else:
                api_success = self._test_urls(self.base_url + '/apicast-tests/ping', 'GET')
            logger.info("=== End API URL Connection Test: %s ===\n",
                        "SUCCESS" if api_success else "FAILURE")
            if upload_success and api_success:
                logger.info("Connectivity tests completed successfully")
                print("See %s for more details." % self.config.logging_file)
            else:
                logger.info("Connectivity tests completed with some errors")
                print("See %s for more details." % self.config.logging_file)
                rc = 1
        except requests.ConnectionError as exc:
            print(exc)
            logger.error('Connectivity test failed! '
                         'Please check your network configuration')
            print('Additional information may be in %s' % self.config.logging_file)
            return 1
        return rc

    def handle_fail_rcs(self, req):
        """
        Bail out if we get a 401 and leave a message
        """

        # attempt to read the HTTP response JSON message
        try:
            logger.log(NETWORK, "HTTP Response Message: %s", req.json()["message"])
        except:
            logger.debug("No HTTP Response message present.")

        # handle specific status codes
        if req.status_code >= 400:
            logger.debug("Debug Information:\nHTTP Status Code: %s",
                        req.status_code)
            logger.debug("HTTP Status Text: %s", req.reason)
            if req.status_code == 401:
                logger.error("Please ensure that the system is registered "
                             "with RHSM for CERT auth, or that correct "
                             "credentials are set in %s for BASIC auth.", self.config.conf)
                logger.log(NETWORK, "HTTP Response Text: %s", req.text)
            if req.status_code == 402:
                # failed registration because of entitlement limit hit
                logger.debug('Registration failed by 402 error.')
                try:
                    logger.error(req.json()["message"])
                except LookupError:
                    logger.error("Got 402 but no message")
                    logger.log(NETWORK, "HTTP Response Text: %s", req.text)
                except:
                    logger.error("Got 402 but no message")
                    logger.log(NETWORK, "HTTP Response Text: %s", req.text)
            if req.status_code == 403 and self.auto_config:
                # Insights disabled in satellite
                rhsm_hostname = urlparse(self.base_url).hostname
                if (rhsm_hostname != 'subscription.rhn.redhat.com' and
                   rhsm_hostname != 'subscription.rhsm.redhat.com'):
                    logger.error('Please enable Insights on Satellite server '
                                 '%s to continue.', rhsm_hostname)
            if req.status_code == 412:
                try:
                    unreg_date = req.json()["unregistered_at"]
                    logger.error(req.json()["message"])
                    write_unregistered_file(unreg_date)
                except LookupError:
                    unreg_date = "412, but no unreg_date or message"
                    logger.log(NETWORK, "HTTP Response Text: %s", req.text)
                except:
                    unreg_date = "412, but no unreg_date or message"
                    logger.log(NETWORK, "HTTP Response Text: %s", req.text)
            if req.status_code == 413:
                logger.error('Archive is too large to upload.')
            if req.status_code == 415:
                logger.error('Invalid content-type.')
            return True
        return False

    def get_satellite5_info(self, branch_info):
        """
        Get remote_leaf for Satellite 5 Managed box
        """
        logger.debug(
            "Remote branch not -1 but remote leaf is -1, must be Satellite 5")
        if os.path.isfile('/etc/sysconfig/rhn/systemid'):
            logger.debug("Found systemid file")
            sat5_conf = ET.parse('/etc/sysconfig/rhn/systemid').getroot()
            leaf_id = None
            for member in sat5_conf.getiterator('member'):
                if member.find('name').text == 'system_id':
                    logger.debug("Found member 'system_id'")
                    leaf_id = member.find('value').find(
                        'string').text.split('ID-')[1]
                    logger.debug("Found leaf id: %s", leaf_id)
                    branch_info['remote_leaf'] = leaf_id
            if leaf_id is None:
                logger.error("Could not determine leaf_id!  Exiting!")
                return False

    def get_branch_info(self):
        """
        Retrieve branch_info from Satellite Server
        """
        # branch_info = None
        # if os.path.exists(constants.cached_branch_info):
        #     # use cached branch info file if less than 5 minutes old
        #     #  (failsafe, should be deleted at end of client run normally)
        #     logger.debug(u'Reading branch info from cached file.')
        #     ctime = datetime.utcfromtimestamp(
        #         os.path.getctime(constants.cached_branch_info))
        #     if datetime.utcnow() < (ctime + timedelta(minutes=5)):
        #         with io.open(constants.cached_branch_info, encoding='utf8', mode='r') as f:
        #             branch_info = json.load(f)
        #         return branch_info
        #     else:
        #         logger.debug(u'Cached branch info is older than 5 minutes.')

        logger.debug(u'Obtaining branch information from %s',
                     self.branch_info_url)
        response = self.get(self.branch_info_url)
        if response.status_code != 200:
            logger.debug("There was an error obtaining branch information.")
            logger.debug(u'Bad status from server: %s', response.status_code)
            logger.debug("Assuming default branch information %s" % constants.default_branch_info)
            return False

        branch_info = response.json()
        logger.debug(u'Branch information: %s', json.dumps(branch_info))

        # Determine if we are connected to Satellite 5
        if ((branch_info[u'remote_branch'] != -1 and
             branch_info[u'remote_leaf'] == -1)):
            self.get_satellite5_info(branch_info)

        # logger.debug(u'Saving branch info to file.')
        # with io.open(constants.cached_branch_info, encoding='utf8', mode='w') as f:
        #     # json.dump is broke in py2 so use dumps
        #     bi_str = json.dumps(branch_info, ensure_ascii=False)
        #     f.write(bi_str)
        self.config.branch_info = branch_info
        return branch_info

    # -LEGACY-
    def create_system(self, new_machine_id=False):
        """
        Create the machine via the API
        """
        client_hostname = determine_hostname()
        machine_id = generate_machine_id(new_machine_id)

        branch_info = self.config.branch_info
        if not branch_info:
            return False

        remote_branch = branch_info['remote_branch']
        remote_leaf = branch_info['remote_leaf']

        data = {'machine_id': machine_id,
                'remote_branch': remote_branch,
                'remote_leaf': remote_leaf,
                'hostname': client_hostname}
        if self.config.display_name is not None:
            data['display_name'] = self.config.display_name
        data = json.dumps(data)
        post_system_url = self.api_url + '/v1/systems'
        logger.debug("POST System: %s", post_system_url)
        logger.debug(data)
        return self.post(post_system_url,
                         headers={'Content-Type': 'application/json'},
                         data=data)

    # -LEGACY-
    def group_systems(self, group_name, systems):
        """
        Adds an array of systems to specified group

        Args:
            group_name: Display name of group
            systems: Array of {'machine_id': machine_id}
        """
        api_group_id = None
        headers = {'Content-Type': 'application/json'}
        group_path = self.api_url + '/v1/groups'
        group_get_path = group_path + ('?display_name=%s' % quote(group_name))

        get_group = self.get(group_get_path)
        if get_group.status_code == 200:
            api_group_id = get_group.json()['id']

        if get_group.status_code == 404:
            # Group does not exist, POST to create
            data = json.dumps({'display_name': group_name})
            post_group = self.post(group_path,
                                   headers=headers,
                                   data=data)
            self.handle_fail_rcs(post_group)
            api_group_id = post_group.json()['id']

        data = json.dumps(systems)
        self.put(group_path +
                 ('/%s/systems' % api_group_id),
                 headers=headers,
                 data=data)

    # -LEGACY-
    # Keeping this function around because it's not private and I don't know if anything else uses it
    def do_group(self):
        """
        Do grouping on register
        """
        group_id = self.config.group
        systems = {'machine_id': generate_machine_id()}
        self.group_systems(group_id, systems)

    # -LEGACY-
    def _legacy_api_registration_check(self):
        '''
        Check registration status through API
        '''
        logger.debug('Checking registration status...')
        machine_id = generate_machine_id()
        try:
            url = self.api_url + '/v1/systems/' + machine_id
            res = self.get(url)
        except requests.ConnectionError:
            # can't connect, run connection test
            logger.error('Connection timed out. Running connection test...')
            self.test_connection()
            return False
        # had to do a quick bugfix changing this around,
        #   which makes the None-False-True dichotomy seem weird
        #   TODO: reconsider what gets returned, probably this:
        #       True for registered
        #       False for unregistered
        #       None for system 404
        self.handle_fail_rcs(res)
        try:
            # check the 'unregistered_at' key of the response
            unreg_status = json.loads(res.content).get('unregistered_at', 'undefined')
            # set the global account number
            self.config.account_number = json.loads(res.content).get('account_number', 'undefined')
        except ValueError:
            # bad response, no json object
            return False
        if unreg_status == 'undefined':
            # key not found, machine not yet registered
            return None
        elif unreg_status is None:
            # unregistered_at = null, means this machine IS registered
            return True
        else:
            # machine has been unregistered, this is a timestamp
            return unreg_status

    def _fetch_system_by_machine_id(self):
        '''
        Get a system by machine ID
        Returns
            dict    system exists in inventory
            False   system does not exist in inventory
            None    error connection or parsing response
        '''
        machine_id = generate_machine_id()
        try:
            # [circus music]
            if self.config.legacy_upload:
                url = self.base_url + '/platform/inventory/v1/hosts?insights_id=' + machine_id
            else:
                url = self.inventory_url + '/hosts?insights_id=' + machine_id
            res = self.get(url)
        except REQUEST_FAILED_EXCEPTIONS as e:
            _api_request_failed(e)
            return None
        try:
            if (self.handle_fail_rcs(res)):
                return None
            res_json = json.loads(res.content)
        except ValueError as e:
            logger.error(e)
            logger.error('Could not parse response body.')
            return None
        if res_json['total'] == 0:
            logger.debug('No hosts found with machine ID: %s', machine_id)
            return False
        return res_json['results']

    def api_registration_check(self):
        '''
            Reach out to the inventory API to check
            whether a machine exists.

            Returns
                True    system exists in inventory
                False   system does not exist in inventory
                None    error connection or parsing response
        '''
        if self.config.legacy_upload:
            return self._legacy_api_registration_check()

        logger.debug('Checking registration status...')
        results = self._fetch_system_by_machine_id()
        if not results:
            return results

        logger.debug('System found.')
        logger.debug('Machine ID: %s', results[0]['insights_id'])
        logger.debug('Inventory ID: %s', results[0]['id'])
        return True

    # -LEGACY-
    def _legacy_unregister(self):
        """
        Unregister this system from the insights service
        """
        machine_id = generate_machine_id()
        try:
            logger.debug("Unregistering %s", machine_id)
            url = self.api_url + "/v1/systems/" + machine_id
            self.delete(url)
            logger.info(
                "Successfully unregistered from the Red Hat Insights Service")
            return True
        except requests.ConnectionError as e:
            logger.debug(e)
            logger.error("Could not unregister this system")
            return False

    def unregister(self):
        """
        Unregister this system from the insights service
        """
        if self.config.legacy_upload:
            return self._legacy_unregister()

        results = self._fetch_system_by_machine_id()
        if not results:
            logger.info('This host could not be found.')
            return False
        try:
            logger.debug("Unregistering host...")
            url = self.inventory_url + "/hosts/" + results[0]['id']
            response = self.delete(url)
            response.raise_for_status()
            logger.info(
                "Successfully unregistered from the Red Hat Insights Service")
            return True
        except (requests.ConnectionError, requests.Timeout, requests.HTTPError) as e:
            logger.debug(e)
            logger.error("Could not unregister this system")
            return False

    # -LEGACY-
    def register(self):
        """
        Register this machine
        """
        client_hostname = determine_hostname()
        # This will undo a blacklist
        logger.debug("API: Create system")
        system = self.create_system(new_machine_id=False)
        if system is False:
            return ('Could not reach the Insights service to register.', '', '', '')

        # If we get a 409, we know we need to generate a new machine-id
        if system.status_code == 409:
            system = self.create_system(new_machine_id=True)
        self.handle_fail_rcs(system)

        logger.debug("System: %s", system.json())

        message = system.headers.get("x-rh-message", "")

        # Do grouping
        if self.config.group is not None:
            self.do_group()

        # Display registration success messasge to STDOUT and logs
        if system.status_code == 201:
            try:
                system_json = system.json()
                machine_id = system_json["machine_id"]
                account_number = system_json["account_number"]
                logger.info("You successfully registered %s to account %s." % (machine_id, account_number))
            except:
                logger.debug('Received invalid JSON on system registration.')
                logger.debug('API still indicates valid registration with 201 status code.')
                logger.debug(system)
                logger.debug(system.json())

        if self.config.group is not None:
            return (message, client_hostname, self.config.group, self.config.display_name)
        elif self.config.display_name is not None:
            return (message, client_hostname, "None", self.config.display_name)
        else:
            return (message, client_hostname, "None", "")

    # -LEGACY-
    def _legacy_upload_archive(self, data_collected, duration):
        '''
        Do an HTTPS upload of the archive
        '''
        file_name = os.path.basename(data_collected)
        try:
            from insights.contrib import magic
            m = magic.open(magic.MAGIC_MIME)
            m.load()
            mime_type = m.file(data_collected)
        except ImportError:
            magic = None
            logger.debug('python-magic not installed, using backup function...')
            from .utilities import magic_plan_b
            mime_type = magic_plan_b(data_collected)

        files = {
            'file': (file_name, open(data_collected, 'rb'), mime_type)}

        upload_url = self.upload_url + '/' + generate_machine_id()

        logger.debug("Uploading %s to %s", data_collected, upload_url)

        headers = {'x-rh-collection-time': str(duration)}
        upload = self.post(upload_url, files=files, headers=headers)

        if upload.status_code in (200, 201):
            the_json = json.loads(upload.text)
        else:
            logger.error("Upload archive failed with status code  %s", upload.status_code)
            return upload
        try:
            self.config.account_number = the_json["upload"]["account_number"]
        except:
            self.config.account_number = None
        logger.debug("Upload duration: %s", upload.elapsed)
        return upload

    def upload_archive(self, data_collected, content_type, duration=None):
        """
        Do an HTTPS Upload of the archive
        """
        if self.config.legacy_upload:
            return self._legacy_upload_archive(data_collected, duration)
        file_name = os.path.basename(data_collected)
        upload_url = self.upload_url
        c_facts = {}

        try:
            c_facts = get_canonical_facts()
        except Exception as e:
            logger.debug('Error getting canonical facts: %s', e)
        if self.config.display_name:
            # add display_name to canonical facts
            c_facts['display_name'] = self.config.display_name
        if self.config.ansible_host:
            # add ansible_host to canonical facts
            c_facts['ansible_host'] = self.config.ansible_host
        if self.config.branch_info:
            c_facts["branch_info"] = self.config.branch_info
            c_facts["satellite_id"] = self.config.branch_info["remote_leaf"]
        c_facts = json.dumps(c_facts)
        logger.debug('Canonical facts collected:\n%s', c_facts)

        files = {
            'file': (file_name, open(data_collected, 'rb'), content_type),
            'metadata': c_facts
        }
        logger.debug('content-type: %s', content_type)
        logger.debug("Uploading %s to %s", data_collected, upload_url)
        upload = self.post(upload_url, files=files, headers={})

        logger.debug('Request ID: %s', upload.headers.get('x-rh-insights-request-id', None))
        if upload.status_code in (200, 202):
            # 202 from platform, no json response
            logger.debug(upload.text)
            # upload = registration on platform
            try:
                write_registered_file()
            except OSError as e:
                if e.errno == errno.EACCES and os.getuid() != 0:
                    # if permissions error as non-root, ignore
                    pass
                else:
                    logger.error('Could not update local registration record: %s', str(e))
        else:
            logger.debug(
                "Upload archive failed with status code %s",
                upload.status_code)
            return upload
        logger.debug("Upload duration: %s", upload.elapsed)
        return upload

    # -LEGACY-
    def _legacy_set_display_name(self, display_name):
        machine_id = generate_machine_id()
        try:
            url = self.api_url + '/v1/systems/' + machine_id

            res = self.get(url)
            old_display_name = json.loads(res.content).get('display_name', None)
            if display_name == old_display_name:
                logger.debug('Display name unchanged: %s', old_display_name)
                return True

            res = self.put(url,
                           headers={'Content-Type': 'application/json'},
                           data=json.dumps(
                               {'display_name': display_name}))
            if res.status_code == 200:
                logger.info('System display name changed from %s to %s',
                            old_display_name,
                            display_name)
                return True
            elif res.status_code == 404:
                logger.error('System not found. '
                             'Please run insights-client --register.')
                return False
            else:
                logger.error('Unable to set display name: %s %s',
                             res.status_code, res.text)
                return False
        except REQUEST_FAILED_EXCEPTIONS + (ValueError,) as e:
            _api_request_failed(e, None)
            # can't connect, run connection test
            return False

    def set_display_name(self, display_name):
        '''
        Set display name of a system independently of upload.
        '''
        if self.config.legacy_upload:
            return self._legacy_set_display_name(display_name)

        system = self._fetch_system_by_machine_id()
        if not system:
            return system
        inventory_id = system[0]['id']

        req_url = self.inventory_url + '/hosts/' + inventory_id
        try:
            res = self.patch(req_url, json={'display_name': display_name})
        except REQUEST_FAILED_EXCEPTIONS as e:
            _api_request_failed(e)
            return False
        if (self.handle_fail_rcs(res)):
            logger.error('Could not update display name.')
            return False
        logger.info('Display name updated to ' + display_name + '.')
        return True

    def set_ansible_host(self, ansible_host):
        '''
        Set Ansible hostname of a system independently of upload.
        '''
        system = self._fetch_system_by_machine_id()
        if not system:
            return system
        inventory_id = system[0]['id']

        req_url = self.inventory_url + '/hosts/' + inventory_id
        try:
            res = self.patch(req_url, json={'ansible_host': ansible_host})
        except REQUEST_FAILED_EXCEPTIONS as e:
            _api_request_failed(e)
            return False
        if (self.handle_fail_rcs(res)):
            logger.error('Could not update Ansible hostname.')
            return False
        logger.info('Ansible hostname updated to ' + ansible_host + '.')
        return True

    def get_diagnosis(self, remediation_id=None):
        '''
            Reach out to the platform and fetch a diagnosis.
            Spirtual successor to --to-json from the old client.
        '''
        # this uses machine id as identifier instead of inventory id
        diag_url = self.base_url + '/remediations/v1/diagnosis/' + generate_machine_id()
        params = {}
        if remediation_id:
            # validate this?
            params['remediation'] = remediation_id
        try:
            res = self.get(diag_url, params=params)
        except (requests.ConnectionError, requests.Timeout) as e:
            _api_request_failed(e)
            return False
        if (self.handle_fail_rcs(res)):
            logger.error('Unable to get diagnosis data: %s %s',
                         res.status_code, res.text)
            return None
        return res.json()

    def _cached_get(self, url):
        '''
            Submits a GET request to @url, caching the result, and returning
            the response body, if any. It makes the response status code opaque
            to the caller.

            Returns: bytes
        '''
        cache = URLCache("/var/cache/insights/cache.dat")

        headers = {}
        item = cache.get(url)
        if item is not None:
            headers["If-None-Match"] = item.etag

        res = self.get(url, headers=headers)

        if res.status_code in [requests.codes.OK, requests.codes.NOT_MODIFIED]:
            if res.status_code == requests.codes.OK:
                if "ETag" in res.headers and len(res.content) > 0:
                    cache.set(url, res.headers["ETag"], res.content)
                    cache.save()
            item = cache.get(url)
            if item is None:
                return res.content
            else:
                return item.content
        else:
            return None

    def get_advisor_report(self):
        '''
            Retrieve advisor report
        '''
        url = self.inventory_url + "/hosts?insights_id=%s" % generate_machine_id()
        content = self._cached_get(url)
        if content is None:
            return None

        host_details = json.loads(content)
        if host_details["total"] < 1:
            _host_not_found()
        if host_details["total"] > 1:
            raise Exception("Error: multiple hosts detected (insights_id = %s)" % generate_machine_id())

        if not os.path.exists("/var/lib/insights"):
            os.makedirs("/var/lib/insights", mode=0o755)

        with open("/var/lib/insights/host-details.json", mode="w+b") as f:
            f.write(content)
            logger.debug("Wrote \"/var/lib/insights/host-details.json\"")

        host_id = host_details["results"][0]["id"]
        url = self.base_url + "/insights/v1/system/%s/reports/" % host_id
        content = self._cached_get(url)
        if content is None:
            return None

        with open("/var/lib/insights/insights-details.json", mode="w+b") as f:
            f.write(content)
            logger.debug("Wrote \"/var/lib/insights/insights-details.json\"")

        return json.loads(content)

    def checkin(self):
        '''
            Sends an ultralight check-in request containing only the Canonical Facts.
        '''
        logger.info("Checking in...")

        try:
            canonical_facts = get_canonical_facts()
        except Exception as e:
            logger.debug('Error getting canonical facts: %s', e)
            logger.debug('Falling back to only machine ID.')
            insights_id = generate_machine_id()
            canonical_facts = {"insights_id": str(insights_id)}

        url = self.inventory_url + "/hosts/checkin"
        logger.debug("Sending check-in request to %s with %s" % (url, canonical_facts))
        try:
            response = self.post(url, headers={"Content-Type": "application/json"}, data=json.dumps(canonical_facts))
            # Change to POST when the API is fixed.
        except REQUEST_FAILED_EXCEPTIONS as exception:
            _api_request_failed(exception)
            return None
        logger.debug("Check-in response status code %d" % response.status_code)

        if response.status_code == requests.codes.CREATED:
            # Remove OK when the API is fixed.
            logger.info("Successfully checked in!")
            return True
        elif response.status_code == requests.codes.NOT_FOUND:
            # Remove BAD_REQUEST when the API is fixed.
            _host_not_found()
        else:
            logger.debug("Check-in response body %s" % response.text)
            raise RuntimeError("Unknown check-in API response")



"""
PackageProvidesJava - command ``/bin/echo {java_command_package}``
==================================================================

.. warning::

    This module is deprecated, please use
    :mod:`insights.parsers.package_provides` instead.

This command reads the output of the pre-command:

``for jp in `/bin/ps auxwww | grep java | grep -v grep| awk '{print $11}' | sort -u`; do echo $jp `readlink -e $jp | xargs rpm -qf`; done``

This command looks for all versions of 'java' running and tries to find the
RPM packages which provide them.  The running command and its package name
are stored as properties ``command`` and ``package`` of the object.

The reason why using above pre_command is that we need to record the links
between running_java_command and package which provides the java command. In
``ps aux`` output, we can only get what java command starts a java
application, instead of java package. Through this way, when there is jdk
bug, we can detect whether a running java application will be affected.

Typical contents of the pre_command::

    /usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64

Parsed result::

    self.command = "/usr/lib/jvm/jre/bin/java"
    self.package = "java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64"

Examples:

    >>> command_package = shared[PackageProvidesJava]
    >>> command_package.command
    '/usr/lib/jvm/jre/bin/java'
    >>> command_package.package
    'java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'

Raises:
    insights.parsers.ParseException: if there is no java application running

Raises:
    insights.parsers.SkipException: if running java command is not provided by package installed through yum or rpm
"""

from insights import parser, CommandParser
from ..parsers import ParseException, SkipException
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.package_provides_java)
class PackageProvidesJava(CommandParser):
    """
    .. warning::

        This Combiner is deprecated, please use
        :class:`insights.parsers.package_provides.PackageProvidesCommand` instead.

    Parse the output of pre_command::

        ``for jp in `/bin/ps auxwww | grep java | grep -v grep| awk '{print $11}' | sort -u`; do echo "$jp `readlink -e $jp | xargs rpm -qf`"; done``.

    Attributes:
        command (str): The java command that starts application.
        package (str): Java package that provides above java command.
    """

    def parse_content(self, content):
        deprecated(
            PackageProvidesJava,
            'Please use the :class:`insights.parsers.package_provides.PackageProvidesCommand` instead.'
        )
        if len(content) == 0:
            raise ParseException("Error: ", 'there is not java application running')
        l = content[0].split()
        if len(l) != 2:
            raise SkipException("Error: ",
                                'current running java command is not provided by package installed through yum or rpm')
        self.command = l[0]
        self.package = l[1]



"""
Dracut module configuration files to build and extend the initramfs image
=========================================================================

This module contains the following parsers:

DracutModuleKdumpCaptureService - file ``/usr/lib/dracut/modules.d/99kdumpbase/kdump-capture.service``
------------------------------------------------------------------------------------------------------
"""

from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.dracut_kdump_capture_service)
class DracutModuleKdumpCaptureService(IniConfigFile):
    """
    Class for parsing the `/usr/lib/dracut/modules.d/99kdumpbase/kdump-capture.service` file.

    .. note::
        Please refer to its super-class :py:class:`insights.core.IniConfigFile`
        for full usage.

    Sample input::

        [Unit]
        Description=Kdump Vmcore Save Service
        After=initrd.target initrd-parse-etc.service sysroot.mount
        Before=initrd-cleanup.service

        [Service]
        Type=oneshot
        ExecStart=/bin/kdump.sh
        StandardInput=null
        StandardOutput=syslog

    Examples:
        >>> type(config)
        <class 'insights.parsers.dracut_modules.DracutModuleKdumpCaptureService'>
        >>> 'Service' in config.sections()
        True
        >>> config.has_option('Service', 'Type')
        True
        >>> config.get('Service', 'Type') == 'oneshot'
        True
    """
    pass



"""
NUMACpus - file ``/sys/devices/system/node/node[0-9]*/cpulist``
===============================================================

This parser will parse the content from cpulist file, from individual
NUMA nodes. This parser will return data in (dict) format.

Sample Content from ``/sys/devices/system/node/node0/cpulist``::

    0-6,14-20

Examples:
    >>> type(numa_cpus_obj)
    <class 'insights.parsers.numa_cpus.NUMACpus'>
    >>> numa_cpus_obj.numa_node_name
    'node0'
    >>> numa_cpus_obj.numa_node_details() == {'numa_node_range': ['0-6', '14-20'], 'total_cpus': 14, 'numa_node_name': 'node0'}
    True
    >>> numa_cpus_obj.numa_node_cpus
    ['0-6', '14-20']
    >>> numa_cpus_obj.total_numa_node_cpus
    14
"""

from insights import Parser, parser, LegacyItemAccess
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.numa_cpus)
class NUMACpus(LegacyItemAccess, Parser):
    """
    Parse `/sys/devices/system/node/node[0-9]*/cpulist` file, return a dict
    which contains total number of CPUs per numa node.
    """

    def parse_content(self, content):
        if (not content) or (not self.file_path):
            raise SkipException("No Contents")

        self.data = {}
        self._cpu_ranges = []
        self.numa_node = self.file_path.rsplit("/")[-2] if self.file_path else None

        for line in content:
            total_cpus = 0
            self._cpu_ranges = line.split(',')
            self.data['numa_node_range'] = self._cpu_ranges
            self.data['numa_node_name'] = self.numa_node
            if '-' in self._cpu_ranges[0]:
                for cpu_range in self._cpu_ranges:
                    lower_num = int(cpu_range.split('-')[0])
                    higher_num = int(cpu_range.split('-')[1])
                    ncpus = (higher_num - lower_num + 1)
                    total_cpus = total_cpus + ncpus
            else:
                total_cpus = len(self._cpu_ranges)
            self.data['total_cpus'] = total_cpus

    @property
    def numa_node_name(self):
        """
        (str): It will return the CPU node name when set, else `None`.
        """
        return self.numa_node if self.numa_node else None

    @property
    def numa_node_cpus(self):
        """
        (list): It will return list of CPUs present under NUMA node when set, else `None`.
        """
        return self._cpu_ranges if self._cpu_ranges else None

    @property
    def total_numa_node_cpus(self):
        """
        (int): It will return total number of CPUs per NUMA node
        """
        return self.data['total_cpus'] if 'total_cpus' in self.data else None

    def numa_node_details(self):
        """
        (dict): it will return the number of CPUs per NUMA, NUMA node name, CPU range, when set, else `None`.
        """
        return self.data if self.data else None



"""
SystemctlStatusAll command ``systemctl status --all``
=====================================================
"""

from insights.core import LogFileOutput
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.systemctl_status_all)
class SystemctlStatusAll(LogFileOutput):
    """
    Class for parsing the output from command ``systemctl status --all``.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`

    Sample log lines::

        * redhat.test.com
            State: degraded
             Jobs: 0 queued
           Failed: 2 units
            Since: Thu 2021-09-23 12:03:43 UTC; 3h 7min ago
           CGroup: /
                   |-user.slice
                   | `-user-1000.slice
                   |   |-user@1000.service

        * proc-sys-fs-binfmt_misc.automount - Arbitrary Executable File Formats File System Automount Point
           Loaded: loaded (/usr/lib/systemd/system/proc-sys-fs-binfmt_misc.automount; static; vendor preset: disabled)
           Active: active (running) since Thu 2021-09-23 12:03:43 UTC; 3h 7min ago
            Where: /proc/sys/fs/binfmt_misc
             Docs: https://www.kernel.org/doc/html/latest/admin-guide/binfmt-misc.html
                   https://www.freedesktop.org/wiki/Software/systemd/APIFileSystems

        Sep 23 15:11:07 redhat.test.com systemd[1]: proc-sys-fs-binfmt_misc.automount: Automount point already active?
        Sep 23 15:11:07 redhat.test.com systemd[1]: proc-sys-fs-binfmt_misc.automount: Got automount request for /proc/sys/fs/binfmt_mis

    Examples:
        >>> "Automount point already active?" in log
        True
    """
    pass



from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.specs import Specs
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.client.apps.malware_detection import MalwareDetectionClient
from insights.client.config import InsightsConfig


class MalwareDetectionSpecs(Specs):
    @datasource(HostContext)
    def malware_detection_app(broker):
        """
        Custom datasource to collects content for malware scanner if a scanner is present on the system
        """

        try:
            # Only run malware-detection if it was passed as an option to insights-client
            insights_config = InsightsConfig().load_all()
            if not (insights_config and hasattr(insights_config, 'app') and insights_config.app == 'malware-detection'):
                raise SkipComponent
            mdc = MalwareDetectionClient(insights_config)
            scan_results = mdc.run()
            if scan_results:
                return DatasourceProvider(content=scan_results, relative_path="malware-detection-results.json")
            else:
                raise SkipComponent
        except Exception:
            raise SkipComponent



# -*- coding: utf-8 -*-

import doctest

import pytest

from insights.parsers import fwupdagent, ParseException
from insights.parsers.fwupdagent import FwupdagentDevices, FwupdagentSecurity
from insights.tests import context_wrap

DEVICES = """
{
  "Devices" : [
    {
      "Name" : "Thunderbolt host controller",
      "DeviceId" : "113494429de0754484e6b93d70e879a913311dd5",
      "Guid" : [
        "ea3c7ac8-d937-568a-aab5-e053bd6af9ce",
        "3d3f953a-3408-56fa-81f3-042d73c59c16",
        "e773c51e-a20c-5b29-9f09-6bb0e0ef7560",
        "f1793149-6c12-5a7f-95db-9fefd116748d"
      ],
      "Summary" : "Unmatched performance for high-speed I/O",
      "Plugin" : "thunderbolt",
      "Protocol" : "com.intel.thunderbolt",
      "Flags" : [
        "internal",
        "updatable",
        "require-ac",
        "supported",
        "registered",
        "dual-image"
      ],
      "Vendor" : "Lenovo",
      "VendorId" : "THUNDERBOLT:0x0109|TBT:0x0109",
      "VendorIds" : [
        "THUNDERBOLT:0x0109",
        "TBT:0x0109"
      ],
      "Version" : "20.00",
      "VersionFormat" : "pair",
      "Icons" : [
        "thunderbolt"
      ],
      "Created" : 1633947746,
      "Releases" : [
        {
          "AppstreamId" : "com.lenovo.ThinkPadN2JTF.firmware",
          "RemoteId" : "lvfs",
          "Summary" : "Lenovo ThinkPad X390/ThinkPad T490s Thunderbolt Firmware",
          "Description" : "<p>Lenovo ThinkPad X390/ThinkPad T490s Thunderbolt Firmware</p><p>Fix Thunderbolt SPI ROM Wear out issue.</p>",
          "Version" : "20.00",
          "Filename" : "fc93d9ff3f7ced7eb11d9bf7d8cd009ce408d7d9",
          "Protocol" : "com.intel.thunderbolt",
          "Checksum" : [
            "915a6657f6972937bcb88868e5d8ce1f1ec9fb85"
          ],
          "License" : "LicenseRef-proprietary",
          "Size" : 253952,
          "Created" : 1583253000,
          "Locations" : [
            "https://fwupd.org/downloads/8eef957c95cb6f534448be1faa7bbfc8702d620f64b757d40ee5e0b6b7094c0e-Lenovo-ThinkPad-X390-SystemFirmware-01.cab"
          ],
          "Uri" : "https://fwupd.org/downloads/8eef957c95cb6f534448be1faa7bbfc8702d620f64b757d40ee5e0b6b7094c0e-Lenovo-ThinkPad-X390-SystemFirmware-01.cab",
          "Homepage" : "http://www.lenovo.com",
          "Vendor" : "Lenovo Ltd."
        }
      ]
    },
    {
      "Name" : "USB3.0 Hub",
      "DeviceId" : "54f0d9041b6c5438c7ff825f5139559c5ca1b222",
      "Guid" : [
        "9429e4c7-f053-51d7-9289-75c4ddb14a97",
        "26f33695-3a3e-5c08-badb-f6141390ebd9",
        "10eb3a15-c177-5810-af53-1963e9200e65",
        "60e0a85e-b245-5c84-ba3b-d5bdb540cd47",
        "d91a45a0-4435-59d6-bb6b-9499f4a793c2",
        "022d2f73-4826-546a-ba0f-62579ea848ea"
      ],
      "Summary" : "USB 3.x Hub",
      "Plugin" : "vli",
      "Protocol" : "com.vli.usbhub",
      "Flags" : [
        "updatable",
        "registered",
        "can-verify",
        "can-verify-image",
        "dual-image",
        "self-recovery",
        "add-counterpart-guids"
      ],
      "Vendor" : "VIA Labs, Inc.",
      "VendorId" : "USB:0x2109",
      "Version" : "3.114",
      "VersionFormat" : "bcd",
      "Icons" : [
        "audio-card"
      ],
      "InstallDuration" : 15,
      "Created" : 1633947743
    }
  ]
}
"""

SECURITY = """
{
  "HostSecurityAttributes" : [
    {
      "AppstreamId" : "org.fwupd.hsi.Kernel.Tainted",
      "HsiResult" : "not-tainted",
      "Name" : "Linux kernel",
      "Uri" : "https://fwupd.github.io/hsi.html#org.fwupd.hsi.Kernel.Tainted",
      "Flags" : [
        "success",
        "runtime-issue"
      ]
    },
    {
      "AppstreamId" : "org.fwupd.hsi.EncryptedRam",
      "HsiLevel" : 4,
      "HsiResult" : "not-supported",
      "Name" : "Encrypted RAM",
      "Uri" : "https://fwupd.github.io/hsi.html#org.fwupd.hsi.EncryptedRam"
    }
  ]
}
"""

SECURITY_ERROR_1 = """
Failed to parse arguments: Unknown option --force
"""

SECURITY_ERROR_2 = """
Command not found

Usage:
  fwupdagent [OPTION…]

  get-devices                       Get all devices and possible releases
  get-updates                       Gets the list of updates for connected hardware
  get-upgrades                      Alias to get-updates

Help Options:
  -h, --help        Show help options

Application Options:
  -v, --verbose     Show extra debugging information

This tool can be used from other tools and from shell scripts.
"""


def test_devices():
    devices = FwupdagentDevices(context_wrap(DEVICES))

    assert len(devices["Devices"]) == 2
    assert devices["Devices"][0]["Name"] == "Thunderbolt host controller"
    assert devices["Devices"][0]["Version"] == "20.00"
    assert devices["Devices"][1]["Name"] == "USB3.0 Hub"
    assert devices["Devices"][1]["Version"] == "3.114"


def test_security():
    security = FwupdagentSecurity(context_wrap(SECURITY))
    assert len(security["HostSecurityAttributes"]) == 2
    assert security["HostSecurityAttributes"][0]["Name"] == "Linux kernel"
    assert security["HostSecurityAttributes"][0]["HsiResult"] == "not-tainted"
    assert security["HostSecurityAttributes"][1]["Name"] == "Encrypted RAM"
    assert security["HostSecurityAttributes"][1]["HsiLevel"] == 4

    with pytest.raises(ParseException):
        FwupdagentSecurity(context_wrap(SECURITY_ERROR_1))

    with pytest.raises(ParseException):
        FwupdagentSecurity(context_wrap(SECURITY_ERROR_2))


def test_doc_examples():
    env = {
        "devices": FwupdagentDevices(context_wrap(DEVICES)),
        "security": FwupdagentSecurity(context_wrap(SECURITY)),
    }
    failed, total = doctest.testmod(fwupdagent, globs=env)
    assert failed == 0



from insights.parsers import fcoeadm_i
from insights.parsers import SkipException, ParseException
from insights.parsers.fcoeadm_i import FcoeadmI
from insights.tests import context_wrap
import pytest
import doctest

FCOEADM_I_57810 = """
        Description:      NetXtreme II BCM57810 10 Gigabit Ethernet
        Revision:         10
        Manufacturer:     Broadcom Corporation
        Serial Number:    2C44FD8F4418
        Driver:           bnx2x 1.712.30-0
        Number of Ports:  1

            Symbolic Name:     bnx2fc (QLogic BCM57810) v2.9.6 over eth8.0-fcoe
            OS Device Name:    host6
            Node Name:         0x50060B0000C26237
            Port Name:         0x50060B0000C26236
            FabricName:        0x0000000000000000
            Speed:             Unknown
            Supported Speed:   1 Gbit, 10 Gbit
            MaxFrameSize:      2048
            FC-ID (Port ID):   0xFFFFFFFF
            State:             Online

            Symbolic Name:     bnx2fc (QLogic BCM57810) v2.9.6 over eth6.0-fcoe
            OS Device Name:    host7
            Node Name:         0x50060B0000C26235
            Port Name:         0x50060B0000C26234
            FabricName:        0x0000000000000000
            Speed:             Unknown
            Supported Speed:   1 Gbit, 10 Gbit
            MaxFrameSize:      2048
            FC-ID (Port ID):   0xFFFFFFFF
            State:             Offline
""".strip()


def test_fcoeadm_i():
    result = FcoeadmI(context_wrap(FCOEADM_I_57810, path='/tmp/fcoeadm_i'))
    assert result.fcoe["Driver"] == "bnx2x 1.712.30-0"
    assert result.fcoe["Revision"] == "10"
    assert result.fcoe["Manufacturer"] == "Broadcom Corporation"
    assert result.fcoe["Serial Number"] == "2C44FD8F4418"
    assert result.fcoe["Number of Ports"] == "1"

    assert result.iface_list == ['eth8.0-fcoe', 'eth6.0-fcoe']
    assert result.nic_list == ['eth8', 'eth6']
    assert result.stat_list == ['Online', 'Offline']
    assert result.get_host_from_nic('eth8') == 'host6'
    assert result.get_stat_from_nic('eth8') == 'Online'

    assert result["Driver"] == "bnx2x 1.712.30-0"
    assert result["Revision"] == "10"
    assert result["Manufacturer"] == "Broadcom Corporation"
    assert result["Serial Number"] == "2C44FD8F4418"
    assert result["Number of Ports"] == "1"

    with pytest.raises(ValueError) as sc12:
        result.get_stat_from_nic('abf371294f')
    assert "is NOT real FCoE port provided" in str(sc12)

    with pytest.raises(ValueError) as sc12:
        result.get_host_from_nic('97hjh38k')
    assert "is NOT real FCoE port provided" in str(sc12)


def test_fcoeadm_i_documentation():
    """
    Here we test the examples in the documentation automatically using
    doctest.  We set up an environment which is similar to what a
    rule writer might see - a 'fcoeadm_i' command that has been
    passed in as a parameter to the rule declaration.
    """
    env = {'fi': FcoeadmI(context_wrap(FCOEADM_I_57810))}
    failed, total = doctest.testmod(fcoeadm_i, globs=env)
    assert failed == 0


def test_fcoeadm_i_exp():
    """
    Here test the examples cause expections
    """

    with pytest.raises(SkipException) as sc:
        FcoeadmI(context_wrap(""))
    assert "Input content is empty" in str(sc)

    with pytest.raises(ParseException) as sc:
        FcoeadmI(context_wrap("ERROR"))
    assert "No useful data parsed in content" in str(sc)



"""
TcpIpStats - file ``/proc/net/snmp``
====================================
The ``TcpIpStats`` class implements the parsing of ``/proc/net/snmp``
file, which contains TCP/IP stats of individual layer.

TcpIpStatsIPV6 - file ``/proc/net/snmp6``
-----------------------------------------
The ``TcpIpStatsIPV6`` class implements the parsing of ``/proc/net/snmp6``
file, which contains TCP/IP stats of individual layer.

"""

from .. import Parser, parser, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.proc_snmp_ipv4)
class TcpIpStats(Parser, LegacyItemAccess):
    """
    Parser for ``/proc/net/snmp`` file.

    Sample input is provided in the *Examples*.

    Examples:

        >>> SNMP_CONTENT = '''
        ... Ip: Forwarding DefaultTTL InReceives InHdrErrors InAddrErrors ForwDatagrams InUnknownProtos InDiscards InDelivers OutRequests OutDiscards OutNoRoutes ReasmTimeout ReasmReqds ReasmOKs ReasmFails FragOKs FragFails FragCreates
        ... Ip: 2 64 43767 0 0 0 0 0 41807 18407 12 73 0 0 0 10 0 0 0
        ... Icmp: InMsgs InErrors InCsumErrors InDestUnreachs InTimeExcds InParmProbs InSrcQuenchs InRedirects InEchos InEchoReps InTimestamps InTimestampReps InAddrMasks InAddrMaskReps OutMsgs OutErrors OutDestUnreachs OutTimeExcds OutParmProbs OutSrcQuenchs OutRedirects OutEchos OutEchoReps OutTimestamps OutTimestampReps OutAddrMasks OutAddrMaskReps
        ... Icmp: 34 0 0 34 0 0 0 0 0 0 0 0 0 0 44 0 44 0 0 0 0 0 0 0 0 0 0
        ... IcmpMsg: InType3 OutType3
        ... IcmpMsg: 34 44
        ... Tcp: RtoAlgorithm RtoMin RtoMax MaxConn ActiveOpens PassiveOpens AttemptFails EstabResets CurrEstab InSegs OutSegs RetransSegs InErrs OutRsts InCsumErrors
        ... Tcp: 1 200 120000 -1 444 0 0 6 7 19269 17050 5 4 234 0
        ... Udp: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors InCsumErrors IgnoredMulti
        ... Udp: 18905 34 0 1348 0 0 0 3565
        ... UdpLite: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors InCsumErrors IgnoredMulti
        ... UdpLite: 0 0 0 0 0 0 0 0
        ... '''.strip()
        >>> from insights.tests import context_wrap
        >>> shared = {TcpIpStats: TcpIpStats(context_wrap(SNMP_CONTENT))}
        >>> stats = shared[TcpIpStats]
        >>> snmp_stats = stats.get("Ip")
        >>> print snmp_stats["DefaultTTL"]
        64
        >>> snmp_stats = stats.get("Udp")
        >>> print snmp_stats["InDatagrams"]
        18905


    Resultant Data::

        {
            'Ip':
                {
                    'FragCreates': 0,
                    'ReasmFails': 10,
                    'Forwarding': 2,
                    'ReasmOKs': 0,
                    'ReasmReqds': 0,
                    'ReasmTimeout': 0,
                    ...
                    ...
                },
            'Icmp':
                {
                    'InRedirects': 0,
                    'InMsgs': 34,
                    'InSrcQuenchs': 0,
                    ...
                    ...
                }
            ...
            ...
        }
    """
    def parse_content(self, content):
        snmp_stats = {}
        data_id = []
        for line in content:
            line_split = line.split()
            if line_split[1].isdigit():
                snmp_stats[line_split[0].replace(":", "")] = dict(zip(data_id, map(int, line_split[1:])))
            else:
                data_id = line_split[1:]
        self.data = snmp_stats


@parser(Specs.proc_snmp_ipv6)
class TcpIpStatsIPV6(Parser, LegacyItemAccess):
    """
    Parser for ``/proc/net/snmp6`` file.

    Sample input is provided in the *Examples*.

    Examples:

        >>> SNMP_CONTENT = '''
        ... Ip6InReceives                   	757
        ... Ip6InHdrErrors                  	0
        ... Ip6InTooBigErrors               	0
        ... Ip6InNoRoutes                   	0
        ... Ip6InAddrErrors                 	0
        ... Ip6InDiscards                       10
        ... Ip6OutForwDatagrams             	0
        ... Ip6OutDiscards                  	0
        ... Ip6OutNoRoutes                  	0
        ... Ip6InOctets                     	579410
        ... Icmp6OutErrors                  	0
        ... Icmp6InCsumErrors               	0
        ...'''.strip()
        >>> from insights.tests import context_wrap
        >>> shared = {TcpIpStatsIPV6: TcpIpStatsIPV6(context_wrap(SNMP_CONTENT))}
        >>> stats = shared[TcpIpStatsIPV6]
        >>> IP6_RX_stats = stats.get("Ip6InReceives")
        >>> print IP6_RX_stats
        757
        >>> IP6_In_Disc = stats.get("Ip6InDiscards")
        >>> print IP6_In_Disc
        10


    Resultant Data::

        {
            'Ip6InReceives': 757,
            'Ip6InHdrErrors': 0,
            'Ip6InTooBigErrors': 0,
            'Ip6InNoRoutes': 0,
            'Ip6InAddrErrors': 0,
            'Ip6InDiscards': 10,
            ...
            ...
        }
    """

    def parse_content(self, content):
        snmp6_stats = {}
        for line in content:
            line_split = line.split()
            snmp6_stats[line_split[0]] = int(line_split[1]) if len(line_split) > 1 and line_split[1] else None
        self.data = snmp6_stats



# -*- coding: utf-8 -*-
"""
RpmOstreeStatus - Command ``/usr/bin/rpm-ostree status --json``
===============================================================

The ``RpmOstreeStatus`` class parses the output of the ``/usr/bin/rpm-ostree status --json`` command.

rpm-ostree is a hybrid image and package system; as the name suggests, it
uses OSTree for the image side, and RPM for the package side. It supports
composing RPMs server-side into an OSTree commit (like an image), and clients
can replicate that bit-for-bit, with fast incremental updates. Additionally,
the hybrid nature comes to the fore with client-side package layering and
overrides.

The status command gives information pertaining to the current deployment in
use. It lists the names and refspecs of all possible deployments in order,
such that the first deployment in the list is the default upon boot.

Sample input data::

    {
    "deployments" : [
        {
        "base-commit-meta" : {
            "rpmostree.inputhash" : "d272136f0a700a049da30520591205fec5474125474a58a4c9a63ecc8243f227"
        },
        "requested-local-packages" : [
        ],
        "base-removals" : [
        ],
        "unlocked" : "none",
        "booted" : true,
        "initramfs-etc" : [
        ],
        "id" : "rhel-f0c0294860db563e5906db8c9f257d2bfebe40c93e0320b0e380b879f545e267.0",
        "osname" : "rhel",
        "origin" : "edge:rhel/8/x86_64/edge",
        "pinned" : false,
        "regenerate-initramfs" : false,
        "base-local-replacements" : [
        ],
        "checksum" : "f0c0294860db563e5906db8c9f257d2bfebe40c93e0320b0e380b879f545e267",
        "requested-base-local-replacements" : [
        ],
        "timestamp" : 1614717652,
        "requested-packages" : [
        ],
        "serial" : 0,
        "packages" : [
        ],
        "gpg-enabled" : false,
        "requested-base-removals" : [
        ]
        }
    ],
    "transaction" : null,
    "cached-update" : null
    }

Examples:
    >>> type(status)
    <class 'insights.parsers.rpm_ostree_status.RpmOstreeStatus'>
    >>> len(status.query.deployments.where("booted", True))
    1
"""
from .. import parser, CommandParser, YAMLParser
from ..parsr.query import from_dict
from insights.specs import Specs


@parser(Specs.rpm_ostree_status)
class RpmOstreeStatus(CommandParser, YAMLParser):
    """
    Class ``RpmOstreeStatus`` parses the output of the ``rpm-ostree status --json`` command.

    Attributes:
        data (dict): The parsed output of the command.
        query (insights.parsr.query.Entry): The queryable object representing
            the data dictionary.
    """
    def parse_content(self, content):
        super(RpmOstreeStatus, self).parse_content(content)
        self.query = from_dict(self.data)



"""
CertificatesEnddate - command ``/usr/bin/openssl x509 -noout -enddate -in path/to/cert/file``
=============================================================================================

This command gets the enddate of the certificate files.

Sample Output::

    /usr/bin/find: '/etc/origin/node': No such file or directory
    /usr/bin/find: '/etc/origin/master': No such file or directory
    notAfter=May 25 16:39:40 2019 GMT
    FileName= /etc/origin/node/cert.pem
    unable to load certificate
    139881193203616:error:0906D066:PEM routines:PEM_read_bio:bad end line:pem_lib.c:802:
    unable to load certificate
    140695459370912:error:0906D06C:PEM routines:PEM_read_bio:no start line:pem_lib.c:703:Expecting: TRUSTED CERTIFICATE
    notAfter=May 25 16:39:40 2019 GMT
    FileName= /etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem
    notAfter=Dec  9 10:55:38 2017 GMT
    FileName= /etc/pki/consumer/cert.pem
    notAfter=Jan  1 04:59:59 2022 GMT
    FileName= /etc/pki/entitlement/3343502840335059594.pem
    notAfter=Aug 31 02:19:59 2017 GMT
    FileName= /etc/pki/consumer/cert.pem
    notAfter=Jan  1 04:59:59 2022 GMT
    FileName= /etc/pki/entitlement/2387590574974617178.pem

Examples:
    >>> type(cert_enddate)
    <class 'insights.parsers.certificates_enddate.CertificatesEnddate'>
    >>> paths = cert_enddate.certificates_path
    >>> '/etc/origin/node/cert.pem' in paths
    True
    >>> cert_enddate.expiration_date('/etc/origin/node/cert.pem').datetime
    datetime.datetime(2019, 5, 25, 16, 39, 40)
    >>> cert_enddate.expiration_date('/etc/origin/node/cert.pem').str
    'May 25 16:39:40 2019'
"""

from datetime import datetime
from collections import namedtuple
from insights import parser, CommandParser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.certificates_enddate)
class CertificatesEnddate(CommandParser, dict):
    """Class to parse the expiration date."""

    ExpirationDate = namedtuple('ExpirationDate', ['str', 'datetime'])
    """namedtuple: contains the expiration date in string and datetime format."""

    def parse_content(self, content):
        """Parse the content of crt files."""
        datestamp = None
        for l in content:
            if datestamp and l.startswith("FileName="):
                self[l.split("=")[-1].strip()] = datestamp
                datestamp = None
            elif l.startswith("notAfter="):
                datestamp = l.split("=")[-1].rsplit(" ", 1)[0]
            else:
                datestamp = None
        if not self:
            raise SkipException("No certification files found.")

    @property
    def data(self):
        """ Set data as property to keep compatibility """
        return self

    @property
    def certificates_path(self):
        """list: Return filepaths in list or []."""
        return self.data.keys() if self.data else []

    def expiration_date(self, path):
        """This will return a namedtuple(['str', 'datetime']) contains the
        expiration date in string and datetime format. If the expiration date
        is unparsable, the ExpirationDate.datetime should be None.

        Args:
            path(str): The certificate file path.

        Returns:
            A ExpirationDate for available path. None otherwise.
        """
        path_date = self.data.get(path)
        if path_date:
            try:
                path_datetime = datetime.strptime(path_date, '%b %d %H:%M:%S %Y')
                return self.ExpirationDate(path_date, path_datetime)
            except Exception:
                return self.ExpirationDate(path_date, None)



import pytest

from insights.combiners.ps import Ps
from insights.parsers.ps import PsEoCmd
from insights.specs.datasources import get_running_commands
from insights.tests import context_wrap

PS_EO_CMD = """
   PID COMMAND
     1 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
     2 [kthreadd]
   988 /usr/sbin/httpd -DFOREGROUND
  1036 /usr/sbin/httpd -DFOREGROUND
  1037 /usr/sbin/httpd -DFOREGROUND
  1038 /usr/sbin/httpd -DFOREGROUND
  1039 /usr/sbin/httpd -DFOREGROUND
  1040 /usr/local/sbin/httpd -DFOREGROUND
 28218 /usr/bin/java TestSleepMethod1
 28219 java TestSleepMethod1
 28240 /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.292.b10-1.el7_9.x86_64/jre/bin/java TestSleepMethod2
333083 /home/user3/apps/pycharm-2021.1.1/jbr/bin/java -classpath /home/user3/apps/pycharm-2021.1.1/lib/bootstrap.jar:/home/user3/apps/pycharm-2021.1.1/lib/util.jar:/home/user3/apps/pycharm-2021.1.1/lib/jdom.jar:/home/user3/apps/pycharm-2021.1.1/lib/log4j.jar:/home/user3/apps/pycharm-2021.1.1/lib/jna.jar -Xms128m -Xmx2048m -XX:ReservedCodeCacheSize=512m -XX:+UseG1GC -XX:SoftRefLRUPolicyMSPerMB=50 -XX:CICompilerCount=2 -XX:+HeapDumpOnOutOfMemoryError -XX:-OmitStackTraceInFastThrow -ea -Dsun.io.useCanonCaches=false -Djdk.http.auth.tunneling.disabledSchemes="" -Djdk.attach.allowAttachSelf=true -Djdk.module.illegalAccess.silent=true -Dkotlinx.coroutines.debug=off -Dsun.tools.attach.tmp.only=true -XX:ErrorFile=/home/user3/java_error_in_pycharm_%p.log -XX:HeapDumpPath=/home/user3/java_error_in_pycharm_.hprof -Didea.vendor.name=JetBrains -Didea.paths.selector=PyCharm2021.1 -Djb.vmOptionsFile=/home/user3/.config/JetBrains/PyCharm2021.1/pycharm64.vmoptions -Didea.platform.prefix=Python com.intellij.idea.Main
"""

PS_EO_CMD_MISSING = """
PID COMMAND
  1 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
  2 [kthreadd]
"""

PS_EO_CMD_EXCEPTION = """
  PID COMMAND
    1 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
    2 [kthreadd]
28218 /usr/bin/java TestSleepMethod1
28219 /exception/java
"""

PS_EO_CMD_ONE = """
   PID COMMAND
     1 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
     2 [kthreadd]
   988 /usr/sbin/httpd -DFOREGROUND
"""


class FakeContext(object):
    def shell_out(self, cmd, split=True, timeout=None, keep_rc=False, env=None, signum=None):
        tmp_cmd = cmd.strip().split()
        if 'exception' in tmp_cmd[-1]:
            raise Exception()
        elif tmp_cmd[-1].startswith('/'):
            return [tmp_cmd[-1], ]
        elif tmp_cmd[-1].endswith('java'):
            return ['/usr/bin/java', ]
        elif tmp_cmd[-1].endswith('httpd'):
            return ['/usr/sbin/httpd', ]

        raise Exception()


def test_get_running_commands_present():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD))
    ps = Ps(None, None, None, None, None, None, pseo)
    assert ps is not None
    ctx = FakeContext()

    results = get_running_commands(ps, ctx, ['httpd'])
    assert set(results) == set(['/usr/sbin/httpd', '/usr/local/sbin/httpd'])

    results = get_running_commands(ps, ctx, ['java'])
    assert set(results) == set([
        '/usr/bin/java',
        '/home/user3/apps/pycharm-2021.1.1/jbr/bin/java',
        '/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.292.b10-1.el7_9.x86_64/jre/bin/java'
    ])

    results = get_running_commands(ps, ctx, ['java', 'httpd'])
    assert set(results) == set([
        '/usr/bin/java',
        '/home/user3/apps/pycharm-2021.1.1/jbr/bin/java',
        '/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.292.b10-1.el7_9.x86_64/jre/bin/java',
        '/usr/sbin/httpd',
        '/usr/local/sbin/httpd'
    ])


def test_get_running_commands_missing():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD_MISSING))
    ps = Ps(None, None, None, None, None, None, pseo)
    assert ps is not None
    ctx = FakeContext()

    results = get_running_commands(ps, ctx, ['httpd', 'java'])
    assert set(results) == set()


def test_get_running_commands_cmd_exception():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD_EXCEPTION))
    ps = Ps(None, pseo, None, None, None, None, pseo)
    assert ps is not None
    ctx = FakeContext()

    results = get_running_commands(ps, ctx, ['httpd', 'java'])
    assert set(results) == set(['/usr/bin/java', ])


def test_get_running_commands_exception():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD_MISSING))
    ps = Ps(None, None, None, None, None, None, pseo)
    assert ps is not None
    ctx = FakeContext()

    with pytest.raises(TypeError):
        get_running_commands(ps, ctx, 'not_a_list')

    with pytest.raises(TypeError):
        get_running_commands(ps, ctx, [])


def test_get_running_commands_one():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD_ONE))
    ps = Ps(None, None, None, None, None, None, pseo)
    assert ps is not None
    ctx = FakeContext()

    results = get_running_commands(ps, ctx, ['httpd'])
    assert set(results) == set(['/usr/sbin/httpd', ])



import doctest
import pytest

from insights.parsers import lspci, SkipException
from insights.parsers.lspci import LsPci, LsPciVmmkn
from insights.tests import context_wrap


LSPCI_0 = """
00:00.0 Host bridge: Intel Corporation 2nd Generation Core Processor Family DRAM Controller (rev 09)
00:02.0 VGA compatible controller: Intel Corporation 2nd Generation Core Processor Family Integrated Graphics Controller (rev 09)
03:00.0 Network controller: Intel Corporation Centrino Advanced-N 6205 [Taylor Peak] (rev 34)
0d:00.0 System peripheral: Ricoh Co Ltd PCIe SDXC/MMC Host Controller (rev 07)
""".strip()


INTEL = """
00:00.0 Host bridge: Intel Corporation 2nd Generation Core Processor Family DRAM Controller (rev 09)
00:02.0 VGA compatible controller: Intel Corporation 2nd Generation Core Processor Family Integrated Graphics Controller (rev 09)
03:00.0 Network controller: Intel Corporation Centrino Advanced-N 6205 [Taylor Peak] (rev 34)
""".strip()

OTHER = """
05:00.0 PCI bridge: Renesas Technology Corp. Device 001d
06:00.0 PCI bridge: Renesas Technology Corp. Device 001d
07:00.0 PCI bridge: Renesas Technology Corp. Device 001a
7f:14.4 System peripheral: Intel Corporation Xeon E5 v3/Core i7 DDRIO (VMSE) 0 & 1 (rev 02)
7f:14.5 System peripheral: Intel Corporation Xeon E5 v3/Core i7 DDRIO (VMSE) 0 & 1 (rev 02)
7f:14.6 System peripheral: Intel Corporation Xeon E5 v3/Core i7 DDRIO (VMSE) 0 & 1 (rev 02)
7f:14.7 System peripheral: Intel Corporation Xeon E5 v3/Core i7 DDRIO (VMSE) 0 & 1 (rev 02)
""".strip()

RENESAS = """
05:00.0 PCI bridge: Renesas Technology Corp. Device 001d
06:00.0 PCI bridge: Renesas Technology Corp. Device 001d
07:00.0 PCI bridge: Renesas Technology Corp. Device 001a
""".strip()


LSPCI_DRIVER_DETAILS = """
00:00.0 Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
00:01.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 1 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:02.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 2 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:03.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 3 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:07.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 7 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:08.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 8 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:09.0 PCI bridge: Intel Corporation 7500/5520/5500/X58 I/O Hub PCI Express Root Port 9 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:0a.0 PCI bridge: Intel Corporation 7500/5520/5500/X58 I/O Hub PCI Express Root Port 10 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:10.0 PIC: Intel Corporation 7500/5520/5500/X58 Physical and Link Layer Registers Port 0 (rev 13)
        Subsystem: Device 0037:0001
00:10.1 PIC: Intel Corporation 7500/5520/5500/X58 Routing and Protocol Layer Registers Port 0 (rev 13)
        Subsystem: Device 0037:0001
00:11.0 PIC: Intel Corporation 7500/5520/5500 Physical and Link Layer Registers Port 1 (rev 13)
        Subsystem: Device 0037:0001
00:11.1 PIC: Intel Corporation 7500/5520/5500 Routing & Protocol Layer Register Port 1 (rev 13)
        Subsystem: Device 0037:0001
00:13.0 PIC: Intel Corporation 7500/5520/5500/X58 I/O Hub I/OxAPIC Interrupt Controller (rev 13)
        Subsystem: Device 0037:0001
00:14.0 PIC: Intel Corporation 7500/5520/5500/X58 I/O Hub System Management Registers (rev 13)
        Subsystem: Device 0037:0001
        Kernel modules: i7core_edac
00:14.1 PIC: Intel Corporation 7500/5520/5500/X58 I/O Hub GPIO and Scratch Pad Registers (rev 13)
        Subsystem: Device 0037:0001
00:14.2 PIC: Intel Corporation 7500/5520/5500/X58 I/O Hub Control Status and RAS Registers (rev 13)
        Subsystem: Device 0037:0001
00:14.3 PIC: Intel Corporation 7500/5520/5500/X58 I/O Hub Throttle Registers (rev 13)
        Subsystem: Device 0037:0001
00:16.0 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.1 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.2 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.3 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.4 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.5 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.6 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:16.7 System peripheral: Intel Corporation 5520/5500/X58 Chipset QuickData Technology Device (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ioatdma
        Kernel modules: ioatdma
00:1a.0 USB controller: Intel Corporation 82801JI (ICH10 Family) USB UHCI Controller #4
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: uhci_hcd
00:1a.1 USB controller: Intel Corporation 82801JI (ICH10 Family) USB UHCI Controller #5
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: uhci_hcd
00:1a.2 USB controller: Intel Corporation 82801JI (ICH10 Family) USB UHCI Controller #6
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: uhci_hcd
00:1a.7 USB controller: Intel Corporation 82801JI (ICH10 Family) USB2 EHCI Controller #2
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ehci_hcd
00:1c.0 PCI bridge: Intel Corporation 82801JI (ICH10 Family) PCI Express Root Port 1
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:1c.4 PCI bridge: Intel Corporation 82801JI (ICH10 Family) PCI Express Root Port 5
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:1c.5 PCI bridge: Intel Corporation 82801JI (ICH10 Family) PCI Express Root Port 6
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:1d.0 USB controller: Intel Corporation 82801JI (ICH10 Family) USB UHCI Controller #1
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: uhci_hcd
00:1d.1 USB controller: Intel Corporation 82801JI (ICH10 Family) USB UHCI Controller #2
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: uhci_hcd
00:1d.2 USB controller: Intel Corporation 82801JI (ICH10 Family) USB UHCI Controller #3
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: uhci_hcd
00:1d.7 USB controller: Intel Corporation 82801JI (ICH10 Family) USB2 EHCI Controller #1
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: ehci_hcd
00:1e.0 PCI bridge: Intel Corporation 82801 PCI Bridge (rev 90)
00:1f.0 ISA bridge: Intel Corporation 82801JIR (ICH10R) LPC Interface Controller
        Subsystem: Cisco Systems Inc Device 0101
        Kernel driver in use: lpc_ich
        Kernel modules: lpc_ich
01:00.0 SCSI storage controller: LSI Logic / Symbios Logic SAS1064ET PCI-Express Fusion-MPT SAS (rev 08)
        Subsystem: Cisco Systems Inc Device 0049
        Kernel driver in use: mptsas
        Kernel modules: mptsas
04:00.0 Fibre Channel: QLogic Corp. ISP2432-based 4Gb Fibre Channel to PCI Express HBA (rev 03)
        Subsystem: Cisco Systems Inc Device 004a
        Kernel driver in use: qla2xxx
        Kernel modules: qla2xxx
04:00.1 Fibre Channel: QLogic Corp. ISP2432-based 4Gb Fibre Channel to PCI Express HBA (rev 03)
        Subsystem: Cisco Systems Inc Device 004a
        Kernel driver in use: qla2xxx
        Kernel modules: qla2xxx
06:00.0 Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
        Subsystem: Cisco Systems Inc Device 004a
        Kernel driver in use: ixgbe
        Kernel modules: ixgbe
06:00.1 Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
        Subsystem: Cisco Systems Inc Device 004a
        Kernel driver in use: ixgbe
        Kernel modules: ixgbe
09:00.0 VGA compatible controller: Matrox Electronics Systems Ltd. MGA G200e [Pilot] ServerEngines (SEP1) (rev 02)
        Subsystem: Cisco Systems Inc Device 0101
""".strip()

LSPCI_DRIVER_DETAILS_2 = """
04:00.0 Fibre Channel: QLogic Corp. ISP2432-based 4Gb Fibre Channel to PCI Express HBA (rev 03)
04:00.1 Fibre Channel: QLogic Corp. ISP2432-based 4Gb Fibre Channel to PCI Express HBA (rev 03)
06:00.1 Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
09:00.0 VGA compatible controller: Matrox Electronics Systems Ltd. MGA G200e [Pilot] ServerEngines (SEP1) (rev 02)
VGA compatible controller: Matrox Electronics Systems Ltd:MGA G200e [Pilot] ServerEngines (SEP1) (rev 02)
""".strip()

LSPCI_DRIVER_DETAILS_3 = """
Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
VGA compatible controller: Matrox Electronics Systems Ltd:MGA G200e [Pilot] ServerEngines (SEP1) (rev 02)
""".strip()

LSPCI_DRIVER_DOC = """
00:00.0 Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)
        Subsystem: Cisco Systems Inc Device 0101
00:01.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 1 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
00:02.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 2 (rev 13)
        Kernel driver in use: pcieport
        Kernel modules: shpchp
03:00.0 Network controller: Intel Corporation Centrino Advanced-N 6205 [Taylor Peak] (rev 34)
        Subsystem: Cisco Systems Inc Device 004a
        Kernel driver in use: ixgbe
        Kernel modules: ixgbe
06:00.0 Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
        Subsystem: Cisco Systems Inc Device 004a
        Kernel driver in use: ixgbe
        Kernel modules: ixgbe
""".strip()

LSPCI_OUTPUT_WITH_BAD_LINES = """
03:00.0 Network controller: Intel Corporation Wireless 7260 (rev bb)
        Subsystem: Intel Corporation Dual Band Wireless-AC 7260

        Kernel driver in use: iwlwifi
# a comment
        Kernel modules: iwlwifi
""".strip()

LSPCI_VMMKN = """
Slot:   00:00.0
Class:  0600
Vendor: 8086
Device: 1237
SVendor:    1af4
SDevice:    1100
Rev:    02

Slot:   00:01.0
Class:  0101
Vendor: 8086
Device: 7010
SVendor:    1af4
SDevice:    1100
ProgIf: 80
Driver: ata_piix
Module: ata_piix
Module: ata_generic

Slot:   00:01.1
Class:  0c03
Vendor: 8086
Device: 7020
SVendor:    1af4
SDevice:    1100
Rev:    01
Driver: uhci_hcd


Slot:   00:03.0
Class:  0200
Vendor: 1af4
Device: 1000
SVendor:    1af4
SDevice:    0001
PhySlot:    3
Driver: virtio-pci
""".strip()


def test_lspci():
    LsPci.token_scan('centrino', 'Centrino')
    lspci_0 = LsPci(context_wrap(LSPCI_0))
    assert [i['raw_message'] for i in lspci_0.get("Intel Corporation")] == INTEL.splitlines()
    assert len(lspci_0.get("Network controller")) == 1
    assert "Centrino Advanced-N 6205" in lspci_0
    assert "0d:00.0" in lspci_0
    other = LsPci(context_wrap(OTHER))
    assert [i['raw_message'] for i in other.get("Renesas Technology Corp.")] == RENESAS.splitlines()
    assert len(other.get("Xeon E5 v3")) == 4
    assert len(other.get("001a")) == 1
    assert "Core i7" in other


def test_lspci_driver():
    lspci_obj = LsPci(context_wrap(LSPCI_DRIVER_DETAILS))
    assert len(lspci_obj.data) == 44
    dev_info = lspci_obj.pci_dev_details('00:01.0')
    assert len(dev_info) == 4
    assert dev_info['Kernel driver in use'] == 'pcieport'
    assert dev_info['Slot'] == '00:01.0'
    assert len(lspci_obj.pci_dev_list) == 44

    lspci_obj = LsPci(context_wrap(LSPCI_DRIVER_DETAILS_2))
    assert len(lspci_obj.data) == 4
    dev_info = lspci_obj.pci_dev_details('04:00.0')
    assert len(dev_info) == 2
    assert dev_info['Slot'] == '04:00.0'
    assert 'Kernel driver in use' not in dev_info
    assert len(lspci_obj.pci_dev_list) == 4

    lspci_obj = LsPci(context_wrap(LSPCI_DRIVER_DETAILS_3))
    assert len(lspci_obj.data) == 0
    assert len(lspci_obj.pci_dev_list) == 0

    output = LsPci(context_wrap(LSPCI_OUTPUT_WITH_BAD_LINES))
    assert len(output.data) == 1
    assert len(output.pci_dev_list) == 1


def test_lspci_vmmkn():
    lspci_vmmkn = LsPciVmmkn(context_wrap(LSPCI_VMMKN))
    assert sorted(lspci_vmmkn.pci_dev_list) == ['00:00.0', '00:01.0', '00:01.1', '00:03.0']
    assert lspci_vmmkn[0].get('Driver') is None
    assert lspci_vmmkn[0].get('Slot') == '00:00.0'
    assert lspci_vmmkn[1].get('Vendor') == '8086'
    assert lspci_vmmkn[1].get('Slot') == '00:01.0'
    assert lspci_vmmkn[1].get('Device') == '7010'
    assert lspci_vmmkn[2].get('Slot') == '00:01.1'
    assert lspci_vmmkn[2].get('SVendor') == '1af4'
    assert lspci_vmmkn[3].get('SDevice') == '0001'
    assert lspci_vmmkn[3].get('Slot') == '00:03.0'
    assert lspci_vmmkn[-1].get('Driver') == 'virtio-pci'
    assert sorted(lspci_vmmkn[1].get('Module')) == sorted(['ata_piix', 'ata_generic'])
    assert lspci_vmmkn[-1].get('Module') is None


def test_lspci_vmmkn_ab():
    with pytest.raises(SkipException):
        LsPciVmmkn(context_wrap(''))

    with pytest.raises(SkipException):
        LsPciVmmkn(context_wrap(' \n '.splitlines()))


def test_doc_examples():
    env = {
            'lspci': LsPci(context_wrap(LSPCI_DRIVER_DOC)),
            'lspci_vmmkn': LsPciVmmkn(context_wrap(LSPCI_VMMKN))
          }
    failed, total = doctest.testmod(lspci, globs=env)
    assert failed == 0



"""
AWSInstanceID
=============

These parsers read the output of commands to collect identify information
from AWS instances.

* ``curl -s http://169.254.169.254/latest/dynamic/instance-identity/document`` and
* ``curl -s http://169.254.169.254/latest/dynamic/instance-identity/pkcs7``
"""
from __future__ import print_function
import json

from insights.parsers import SkipException, ParseException
from insights import parser, CommandParser
from insights.specs import Specs


@parser(Specs.aws_instance_id_doc)
class AWSInstanceIdDoc(CommandParser, dict):
    """
    Class for parsing the AWS Instance Identity Document returned by the command::

        curl -s http://169.254.169.254/latest/dynamic/instance-identity/document

    Typical output of this command is::

        {
            "devpayProductCodes" : null,
            "marketplaceProductCodes" : [ "1abc2defghijklm3nopqrs4tu" ],
            "availabilityZone" : "us-west-2b",
            "privateIp" : "10.158.112.84",
            "version" : "2017-09-30",
            "instanceId" : "i-1234567890abcdef0",
            "billingProducts" : [ "bp-6ba54002" ],
            "instanceType" : "t2.micro",
            "accountId" : "123456789012",
            "imageId" : "ami-5fb8c835",
            "pendingTime" : "2016-11-19T16:32:11Z",
            "architecture" : "x86_64",
            "kernelId" : null,
            "ramdiskId" : null,
            "region" : "us-west-2"
        }

    Raises:
        SkipException: When content is empty or cannot be parsed.
        ParseException: When type cannot be recognized.

    Attributes:
        dict: Parser object is a dictionary that is a direct translation of the input key:value pairs.
        json (str): Input in JSON string format.

    Examples:
        >>> print(aws_id_doc['billingProducts'][0])
        bp-6ba54002
        >>> 'version' in aws_id_doc
        True
        >>> print(aws_id_doc['version'])
        2017-09-30
    """

    def parse_content(self, content):
        """Parse output of command."""
        if not content or 'curl: ' in content[0]:
            raise SkipException()

        # Just in case curl stats are present in data
        startline = 0
        for l in content:
            if l.strip().startswith('{'):
                break
            startline += 1

        self.json = '\n'.join([l.rstrip() for l in content[startline:]])

        try:
            doc_values = json.loads(self.json)
            self.update(doc_values)
        except ValueError as e:
            raise ParseException('Failed to parse json with error: %s', str(e))


@parser(Specs.aws_instance_id_pkcs7)
class AWSInstanceIdPkcs7(CommandParser):
    """
    Class for parsing the AWS Instance Identity PKCS7 signature returned by the command::

        curl -s http://169.254.169.254/latest/dynamic/instance-identity/pkcs7

    Typical output of this command is::

        MIICiTCCAfICCQD6m7oRw0uXOjANBgkqhkiG9w0BAQUFADCBiDELMAkGA1UEBhMC
        VVMxCzAJBgNVBAgTAldBMRAwDgYDVQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6
        b24xFDASBgNVBAsTC0lBTSBDb25zb2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAd
        BgkqhkiG9w0BCQEWEG5vb25lQGFtYXpvbi5jb20wHhcNMTEwNDI1MjA0NTIxWhcN
        MTIwNDI0MjA0NTIxWjCBiDELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMRAwDgYD
        VQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6b24xFDASBgNVBAsTC0lBTSBDb25z
        b2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAdBgkqhkiG9w0BCQEWEG5vb25lQGFt
        YXpvbi5jb20wgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMaK0dn+a4GmWIWJ
        21uUSfwfEvySWtC2XADZ4nB+BLYgVIk60CpiwsZ3G93vUEIO3IyNoH/f0wYK8m9T
        rDHudUZg3qX4waLG5M43q7Wgc/MbQITxOUSQv7c7ugFFDzQGBzZswY6786m86gpE
        Ibb3OhjZnzcvQAaRHhdlQWIMm2nrAgMBAAEwDQYJKoZIhvcNAQEFBQADgYEAtCu4
        nUhVVxYUntneD9+h8Mg9q6q+auNKyExzyLwaxlAoo7TJHidbtS4J5iNmZgXL0Fkb
        FFBjvSfpJIlJ00zbhNYS5f6GuoEDmFJl0ZxBHjJnyp378OD8uTs7fLvjx79LjSTb
        NYiytVbZPQUQ5Yaxu2jXnimvw3rrszlaEXAMPLE

    Raises:
        SkipException: When content is empty or cannot be parsed.

    Attributes:
        signature (str): PKCS7 signature string including header and footer.

    Examples:
        >>> aws_id_sig.signature.startswith('-----BEGIN PKCS7-----\\nMIICiTCCAfICCQD6m7oRw0uXOjANBgkqhkiG9w0BAQUFADCBiDELMAkGA1UEBhMC\\n')
        True
        >>> aws_id_sig.signature.endswith('NYiytVbZPQUQ5Yaxu2jXnimvw3rrszlaEXAMPLE\\n-----END PKCS7-----')
        True
    """
    def parse_content(self, content):
        """Parse output of command."""
        if not content or 'curl: ' in content[0]:
            raise SkipException()

        # Just in case curl stats are present in data
        startline = 0
        for l in content:
            if ' ' not in l.strip() and len(l.strip()) > 0:
                break
            startline += 1

        self.signature = '-----BEGIN PKCS7-----\n' + '\n'.join([l.rstrip() for l in content[startline:]]) + "\n-----END PKCS7-----"



"""
CpuVulnsAll - combiner for CPU vulnerabilities
==============================================

This combiner provides an interface to CPU vulnerabilities parsers for cpu vulnerabilities
"""

from insights.core.plugins import combiner
from insights.parsers.cpu_vulns import CpuVulns
from insights.parsers import SkipComponent


@combiner(CpuVulns)
class CpuVulnsAll(dict):
    """
    Class to capsulate the parsers of cpu_vulns, files information will be
    stored in a list of dictionaries, each dictionary is for one file, the
    dictionary key is the file name, dictionary value is the file content.

    Sample output for files:
        ``/sys/devices/system/cpu/vulnerabilities/spectre_v1``:
            Mitigation: Load fences

        ``/sys/devices/system/cpu/vulnerabilities/meltdown``:
            Mitigation: PTI

    Examples:
        >>> type(cvb)
        <class 'insights.combiners.cpu_vulns_all.CpuVulnsAll'>
        >>> list(cvb.keys())
        ['meltdown', 'spectre_v1']
        >>> cvb['meltdown']
        'Mitigation: PTI'

    Raises:
        SkipComponent: Not available data

    """

    def __init__(self, cpu_vulns):
        for cv in cpu_vulns:
            self.update({cv.file_name: cv.value}) if cv.file_name else None
        if len(self) == 0:
            raise SkipComponent('Not available data')



from insights.core import dr


class stage(dr.ComponentType):
    metadata = {"description": "A processing stage."}


@stage()
def report():
    return "this is a regular report"


@stage(metadata={"description": "Reports stuff about things."})
def special_report():
    return "this is a special report"


def test_component_metadata():
    msg = "A processing stage."
    special_msg = "Reports stuff about things."
    assert dr.get_metadata(report).get("description") == msg
    assert dr.get_metadata(special_report).get("description") == special_msg



from insights.parsr.examples.corosync_conf import loads

DATA = r"""
# Please read the corosync.conf.5 manual page
totem {
    version: 2

    # crypto_cipher and crypto_hash: Used for mutual node authentication.
    # If you choose to enable this, then do remember to create a shared
    # secret with "corosync-keygen".
    # enabling crypto_cipher, requires also enabling of crypto_hash.
    crypto_cipher: none
    crypto_hash: none

    # interface: define at least one interface to communicate
    # over. If you define more than one interface stanza, you must
    # also set rrp_mode.
    interface {
                # Rings must be consecutively numbered, starting at 0.
        ringnumber: 0
        # This is normally the *network* address of the
        # interface to bind to. This ensures that you can use
        # identical instances of this configuration file
        # across all your cluster nodes, without having to
        # modify this option.
        bindnetaddr: 192.168.1.0
        # However, if you have multiple physical network
        # interfaces configured for the same subnet, then the
        # network address alone is not sufficient to identify
        # the interface Corosync should bind to. In that case,
        # configure the *host* address of the interface
        # instead:
        # bindnetaddr: 192.168.1.1
        # When selecting a multicast address, consider RFC
        # 2365 (which, among other things, specifies that
        # 239.255.x.x addresses are left to the discretion of
        # the network administrator). Do not reuse multicast
        # addresses across multiple Corosync clusters sharing
        # the same network.
        mcastaddr: 239.255.1.1
        # Corosync uses the port you specify here for UDP
        # messaging, and also the immediately preceding
        # port. Thus if you set this to 5405, Corosync sends
        # messages over UDP ports 5405 and 5404.
        mcastport: 5405
        # Time-to-live for cluster communication packets. The
        # number of hops (routers) that this ring will allow
        # itself to pass. Note that multicast routing must be
        # specifically enabled on most network routers.
        ttl: 1
    }
}

logging {
    # Log the source file and line where messages are being
    # generated. When in doubt, leave off. Potentially useful for
    # debugging.
    fileline: off
    # Log to standard error. When in doubt, set to no. Useful when
    # running in the foreground (when invoking "corosync -f")
    to_stderr: no
    # Log to a log file. When set to "no", the "logfile" option
    # must not be set.
    to_logfile: yes
    logfile: /var/log/cluster/corosync.log
    # Log to the system log daemon. When in doubt, set to yes.
    to_syslog: yes
    # Log debug messages (very verbose). When in doubt, leave off.
    debug: off
    # Log messages with time stamps. When in doubt, set to on
    # (unless you are only logging to syslog, where double
    # timestamps can be annoying).
    timestamp: on
    logger_subsys {
        subsys: QUORUM
        debug: off
    }
}

quorum {
    # Enable and configure quorum subsystem (default: off)
    # see also corosync.conf.5 and votequorum.5
    #provider: corosync_votequorum
}
""".strip()

COROSYNC_CONF = """
totem {
    version: 2
    secauth: off
    cluster_name: tripleo_cluster
    transport: udpu
    token: 10000
}

nodelist {
    node {
        ring0_addr: overcloud-controller-0
        nodeid: 1
    }

    node {
        ring0_addr: overcloud-controller-1
        nodeid: 2
    }

    node {
        ring0_addr: overcloud-controller-2
        nodeid: 3
    }
}

quorum {
    provider: corosync_votequorum
}

logging {
    to_logfile: yes
    logfile: /var/log/cluster/corosync.log
    to_syslog: yes
}
""".strip()


def test_corosync_data():
    res = loads(DATA)
    assert res["totem"]["version"].value == 2
    assert res["totem"]["interface"]["bindnetaddr"].value == "192.168.1.0"
    assert len(res["quorum"]) == 1


def test_corosync_conf():
    conf = loads(COROSYNC_CONF)
    assert conf['totem']['token'][0].value == 10000
    assert conf['quorum']['provider'][0].value == 'corosync_votequorum'
    assert conf['nodelist']['node']['nodeid'][-1].value == 3



from insights.contrib.soscleaner import SOSCleaner

from mock.mock import Mock
from pytest import mark


def _soscleaner():
    soscleaner = SOSCleaner()
    soscleaner.logger = Mock()
    return soscleaner


@mark.parametrize(("line", "expected"), [
    ("radius_ip_1=10.0.0.1", "radius_ip_1=10.230.230.1"),
    (
        (
            "        inet 10.0.2.15"
            "  netmask 255.255.255.0"
            "  broadcast 10.0.2.255"
        ),
        (
            "        inet 10.230.230.1"
            "  netmask 10.230.230.2"
            "  broadcast 10.230.230.3"
        ),
    ),
    (
        "radius_ip_1=10.0.0.100-10.0.0.200",
        "radius_ip_1=10.230.230.1-10.230.230.2",
    ),
])
def test_sub_ip_match(line, expected):
    soscleaner = _soscleaner()
    actual = soscleaner._sub_ip(line)
    assert actual == expected


@mark.xfail
@mark.parametrize(("line",), [
    (
        "{\"name\":\"shadow-utils\","
        "\"epoch\":\"2\","
        "\"version\":\"4.1.5.1\","
        "\"release\":\"5.el6\","
        "\"arch\":\"x86_64\","
        "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
        "\"buildtime\":\"1455012203\","
        "\"vendor\":\"Red Hat, Inc.\","
        "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
        "\"sigpgp\":"
        "\"RSA/8, "
        "Tue 08 Mar 2016 11:15:08 AM CET, "
        "Key ID 199e2f91fd431d51\"}",
    )
])
def test_sub_ip_no_match(line):
    soscleaner = _soscleaner()
    actual = soscleaner._sub_ip(line)
    assert actual == line



import pytest
from insights.parsers import SkipException
from insights.parsers import multipath_conf
from insights.parsr.query import first, last
from insights.tests import context_wrap


CONF = """
blacklist {
       device {
               vendor  "IBM"
               product "3S42"       #DS4200 Product 10
       }
       device {
               vendor  "HP"
               product "*"
       }
}""".strip()


MULTIPATH_CONF_INFO = """
defaults {
       udev_dir                /dev
       path_selector           "round-robin 0"
       user_friendly_names     yes
}

multipaths {
       multipath {
               alias                   yellow
               path_grouping_policy    multibus
       }
       multipath {
               wwid                    1DEC_____321816758474
               alias                   red
       }
}

devices {
       device {
               path_selector           "round-robin 0"
               no_path_retry           queue
       }
       device {
               vendor                  "COMPAQ  "
               path_grouping_policy    multibus
       }
}

blacklist {
       wwid 26353900f02796769
       devnode "^hd[a-z]"
}

""".strip()

INPUT_EMPTY = ''


def test_multipath_conf():
    multipath_conf_info = multipath_conf.MultipathConf(context_wrap(MULTIPATH_CONF_INFO))
    assert multipath_conf_info.get('defaults').get('udev_dir') == '/dev'
    assert multipath_conf_info.get('defaults').get('path_selector') == 'round-robin 0'
    assert multipath_conf_info.get('multipaths')[1].get('alias') == 'red'
    assert multipath_conf_info.get('devices')[0].get('no_path_retry') == 'queue'
    assert multipath_conf_info.get('blacklist').get('devnode') == '^hd[a-z]'


def test_multipath_conf_initramfs():
    multipath_conf_initramfs = multipath_conf.MultipathConfInitramfs(context_wrap(MULTIPATH_CONF_INFO))
    assert multipath_conf_initramfs.get('defaults').get('udev_dir') == '/dev'
    assert multipath_conf_initramfs.get('defaults').get('path_selector') == 'round-robin 0'
    assert multipath_conf_initramfs.get('multipaths')[1].get('alias') == 'red'
    assert multipath_conf_initramfs.get('devices')[0].get('no_path_retry') == 'queue'
    assert multipath_conf_initramfs.get('blacklist').get('devnode') == '^hd[a-z]'


def test_multipath_conf_trees():
    for c in (multipath_conf.MultipathConfTree,
              multipath_conf.MultipathConfTreeInitramfs):
        conf = c(context_wrap(CONF))
        assert len(conf["blacklist"]) == 1
        assert len(conf["blacklist"]["device"]) == 2

        assert len(conf["blacklist"]["device"]["vendor"]) == 2
        assert len(conf["blacklist"]["device"]["product"]) == 2

        assert conf["blacklist"]["device"]["vendor"][first].value == "IBM"
        assert conf["blacklist"]["device"]["vendor"][last].value == "HP"

        assert conf["blacklist"]["device"]["product"][first].value == "3S42"
        assert conf["blacklist"]["device"]["product"][last].value == "*"


def test_empty_multipath_conf_tree():
    with pytest.raises(SkipException) as e_info:
        multipath_conf.MultipathConfTree(context_wrap(INPUT_EMPTY))
    assert "Empty content." in str(e_info.value)

    with pytest.raises(SkipException) as e_info:
        multipath_conf.MultipathConfTreeInitramfs(context_wrap(INPUT_EMPTY))
    assert "Empty content." in str(e_info.value)


def test_empty_multipath_conf():
    with pytest.raises(SkipException) as e_info:
        multipath_conf.MultipathConfParser(context_wrap(INPUT_EMPTY))
    assert "Empty content." in str(e_info.value)



from insights.tests import context_wrap
from insights.parsers.sysconfig import SshdSysconfig

SSHD_SYSCONFIG = """
# Configuration file for the sshd service.

# The server keys are automatically generated if they are missing.
# To change the automatic creation, adjust sshd.service options for
# example using  systemctl enable sshd-keygen@dsa.service  to allow creation
# of DSA key or  systemctl mask sshd-keygen@rsa.service  to disable RSA key
# creation.

# System-wide crypto policy:
# To opt-out, uncomment the following line
# CRYPTO_POLICY=
CRYPTO_POLICY=
""".strip()


def test_sysconfig_sshd():
    result = SshdSysconfig(context_wrap(SSHD_SYSCONFIG))
    assert result["CRYPTO_POLICY"] == ''
    assert result.get("CRYPTO_POLICY") == ''
    assert result.get("OPTIONS1") is None
    assert "OPTIONS1" not in result
    assert "CRYPTO_POLICY" in result



from insights.parsers.package_provides_java import PackageProvidesJava
from insights.combiners.package_provides_java import PackageProvidesJavaAll
from insights.tests import context_wrap

PACKAGE_COMMAND_MATCH_1 = """
/usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64
"""
PACKAGE_COMMAND_MATCH_2 = """
/usr/lib/jvm/java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64
"""

PACKAGE_COMMAND_MATCH_3 = """
java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64
"""


def test_packages_provide_java():
    pack1 = PackageProvidesJava(context_wrap(PACKAGE_COMMAND_MATCH_1))
    pack2 = PackageProvidesJava(context_wrap(PACKAGE_COMMAND_MATCH_2))
    pack3 = PackageProvidesJava(context_wrap(PACKAGE_COMMAND_MATCH_3))
    result = PackageProvidesJavaAll([pack1, pack2, pack3])
    assert sorted(result.running_javas) == sorted(
        ['/usr/lib/jvm/jre/bin/java',
         '/usr/lib/jvm/java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64/bin/java', 'java'])
    assert result[
               "/usr/lib/jvm/java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64/bin/java"] == "java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64"
    assert result.get_package("/usr/lib/jvm/jre/bin/java") == "java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64"
    assert result.get("/usr/lib/jvm/jre/bin/java") == "java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64"
    assert result.get_package("/usr/lib/jre/bin/java") is None
    assert result.get("/usr/lib/jre/bin/java") is None



import doctest

from insights.parsers import systemctl_status_all
from insights.parsers.systemctl_status_all import SystemctlStatusAll
from insights.tests import context_wrap

SYSTEMCTLSTATUSALL_ERROR_LOG = """
* redhat.test.com
    State: degraded
     Jobs: 0 queued
   Failed: 2 units
    Since: Thu 2021-09-23 12:03:43 UTC; 3h 7min ago
   CGroup: /
           |-user.slice
           | `-user-1000.slice
           |   |-user@1000.service

* proc-sys-fs-binfmt_misc.automount - Arbitrary Executable File Formats File System Automount Point
   Loaded: loaded (/usr/lib/systemd/system/proc-sys-fs-binfmt_misc.automount; static; vendor preset: disabled)
   Active: active (running) since Thu 2021-09-23 12:03:43 UTC; 3h 7min ago
    Where: /proc/sys/fs/binfmt_misc
     Docs: https://www.kernel.org/doc/html/latest/admin-guide/binfmt-misc.html
           https://www.freedesktop.org/wiki/Software/systemd/APIFileSystems

Sep 23 15:11:07 redhat.test.com systemd[1]: proc-sys-fs-binfmt_misc.automount: Automount point already active?
Sep 23 15:11:07 redhat.test.com systemd[1]: proc-sys-fs-binfmt_misc.automount: Got automount request for /proc/sys/fs/binfmt_mis
""".strip()


def test_error_log():
    log = SystemctlStatusAll(context_wrap(SYSTEMCTLSTATUSALL_ERROR_LOG))
    assert 19 == len(log.lines)
    assert "Automount point already active?" in log


def test_crio_conf_documentation():
    failed_count, tests = doctest.testmod(
        systemctl_status_all,
        globs={'log': SystemctlStatusAll(context_wrap(SYSTEMCTLSTATUSALL_ERROR_LOG))}
    )
    assert failed_count == 0
